@@ -16,7 +16,7 @@ static inline void scat(char *dest, const char *src)
 }
 
 RtData::RtData(void)
-    :loc(NULL), loc_size(0), obj(NULL), matches(0)
+    :loc(NULL), loc_size(0), obj(NULL), matches(0), message(NULL)
 {}
 
 void RtData::reply(const char *path, const char *args, ...)
@@ -42,6 +42,9 @@ void RtData::broadcast(const char *path, const char *args, ...)
 void RtData::broadcast(const char *msg)
 {reply(msg);};
 
+void RtData::forward(const char *rational)
+{}
+
 void metaiterator_advance(const char *&title, const char *&value)
 {
     if(!title || !*title) {
@@ -373,6 +376,8 @@ static ivec_t find_remap(words_t &strs, ivec_t &pos, ivec_t &assoc)
 
 static void generate_minimal_hash(std::vector<std::string> str, Port_Matcher &pm)
 {
+    if(str.empty())
+        return;
     pm.pos   = find_pos(str);
     if(pm.pos.empty()) {
         fprintf(stderr, "rtosc: Failed to generate minimal hash\n");
@@ -413,14 +418,9 @@ static void generate_minimal_hash(Ports &p, Port_Matcher &pm)
 }
 
 Ports::Ports(std::initializer_list<Port> l)
-    :ports(l), impl(new Port_Matcher)
+    :ports(l), impl(NULL)
 {
-    generate_minimal_hash(*this, *impl);
-    impl->enump = new bool[ports.size()];
-    for(int i=0; i<(int)ports.size(); ++i)
-        impl->enump[i] = strchr(ports[i].name, '#');
-
-    elms = ports.size();
+    refreshMagic();
 }
 
 Ports::~Ports()
@@ -433,9 +433,20 @@ Ports::~Ports()
 #define __builtin_expect(a,b) a
 #endif
 
-void Ports::dispatch(const char *m, rtosc::RtData &d) const
+void Ports::dispatch(const char *m, rtosc::RtData &d, bool base_dispatch) const
 {
     void *obj = d.obj;
+
+    //handle the first dispatch layer
+    if(base_dispatch) {
+        d.matches = 0;
+        d.message = m;
+        if(m && *m == '/')
+            m++;
+        if(d.loc)
+            d.loc[0] = 0;
+    }
+
     //simple case
     if(!d.loc || !d.loc_size) {
         for(const Port &port: ports) {
@@ -501,8 +512,14 @@ void Ports::dispatch(const char *m, rtosc::RtData &d) const
             for(auto p:impl->pos)
                 if(p < (int)len)
                     t += impl->assoc[m[p]];
-            if(t >= (int)impl->remap.size())
+            if(t >= (int)impl->remap.size() && !default_handler)
                 return;
+            else if(t >= (int)impl->remap.size() && default_handler) {
+                d.matches++;
+                default_handler(m,d), d.obj = obj;
+                return;
+            }
+
             int port_num = impl->remap[t];
 
             //Verify the chosen port is correct
@@ -531,6 +548,9 @@ void Ports::dispatch(const char *m, rtosc::RtData &d) const
 
                 //Remove the rest of the path
                 old_end[0] = '\0';
+            } else if(default_handler) {
+                d.matches++;
+                default_handler(m,d), d.obj = obj;
             }
         }
     }
@@ -575,6 +595,128 @@ const Port *Ports::apropos(const char *path) const
     return NULL;
 }
 
+static bool parent_path_p(char *read, char *start)
+{
+    if(read-start<2)
+        return false;
+    return read[0]=='.' && read[-1]=='.' && read[-2]=='/';
+}
+
+static void read_path(char *&r, char *start)
+{
+    while(1)
+    {
+        if(r<start)
+            break;
+        bool doBreak = *r=='/';
+        r--;
+        if(doBreak)
+            break;
+    }
+}
+
+static void move_path(char *&r, char *&w, char *start)
+{
+    while(1)
+    {
+        if(r<start)
+            break;
+        bool doBreak = *r=='/';
+        *w-- = *r--;
+        if(doBreak)
+            break;
+    }
+}
+
+
+char *Ports::collapsePath(char *p)
+{
+    //obtain the pointer to the last non-null char
+    char *p_end = p;
+    while(*p_end) p_end++;
+    p_end--;
+
+    //number of subpaths to consume
+    int consuming = 0;
+
+    char *write_pos = p_end;
+    char *read_pos = p_end;
+    while(read_pos >= p) {
+        //per path chunk either
+        //(1) find a parent ref and inc consuming
+        //(2) find a normal ref and consume
+        //(3) find a normal ref and write through
+        bool ppath = parent_path_p(read_pos, p);
+        if(ppath) {
+            read_path(read_pos, p);
+            consuming++;
+        } else if(consuming) {
+            read_path(read_pos, p);
+            consuming--;
+        } else
+            move_path(read_pos, write_pos, p);
+    }
+    //return last written location, not next to write
+    return write_pos+1;
+};
+
+void Ports::refreshMagic()
+{
+    delete impl;
+    impl = new Port_Matcher;
+    generate_minimal_hash(*this, *impl);
+    impl->enump = new bool[ports.size()];
+    for(int i=0; i<(int)ports.size(); ++i)
+        impl->enump[i] = strchr(ports[i].name, '#');
+
+    elms = ports.size();
+}
+
+ClonePorts::ClonePorts(const Ports &ports_,
+        std::initializer_list<ClonePort> c)
+    :Ports({})
+{
+    for(auto &to_clone:c) {
+        const Port *clone_port = NULL;
+        for(auto &p:ports_.ports)
+            if(!strcmp(p.name, to_clone.name))
+                clone_port = &p;
+        if(!clone_port && strcmp("*", to_clone.name)) {
+            fprintf(stderr, "Cannot find a clone port for '%s'\n",to_clone.name);
+            assert(false);
+        }
+
+        if(clone_port) {
+            ports.push_back({clone_port->name, clone_port->metadata,
+                    clone_port->ports, to_clone.cb});
+        } else {
+            default_handler = to_clone.cb;
+        }
+    }
+
+    refreshMagic();
+}
+MergePorts::MergePorts(std::initializer_list<const rtosc::Ports*> c)
+    :Ports({})
+{
+    //XXX TODO remove duplicates in some sane and documented way
+    //e.g. repeated ports override and remove older ones
+    for(auto *to_clone:c) {
+        assert(to_clone);
+        for(auto &p:to_clone->ports) {
+            bool already_there = false;
+            for(auto &pp:ports)
+                if(!strcmp(pp.name, p.name))
+                    already_there = true;
+
+            if(!already_there)
+                ports.push_back(p);
+        }
+    }
+
+    refreshMagic();
+}
+
 void rtosc::walk_ports(const Ports *base,
                        char         *name_buffer,
                        size_t        buffer_size,
@@ -670,7 +812,7 @@ void walk_ports2(const rtosc::Ports *base,
 
                 //for(unsigned i=0; i<max; ++i)
                 {
-                    sprintf(pos,"[0,%d]",max);
+                    sprintf(pos,"[0,%d]",max-1);
 
                     //Ensure the result is a path
                     if(strrchr(name_buffer, '/')[1] != '/')
@@ -697,7 +839,7 @@ void walk_ports2(const rtosc::Ports *base,
 
                 //for(unsigned i=0; i<max; ++i)
                 {
-                    sprintf(pos,"[0,%d]",max);
+                    sprintf(pos,"[0,%d]",max-1);
 
                     //Apply walker function
                     walker(&p, name_buffer, data);
@@ -724,60 +866,180 @@ static void units(std::ostream &o, const char *u)
     o << " units=\"" << u << "\"";
 }
 
+using std::ostream;
+using std::string;
+static int enum_min(Port::MetaContainer meta)
+{
+    int min = 0;
+    for(auto m:meta)
+        if(strstr(m.title, "map "))
+            min = atoi(m.title+4);
+
+    for(auto m:meta)
+        if(strstr(m.title, "map "))
+            min = min>atoi(m.title+4) ? atoi(m.title+4) : min;
+
+    return min;
+}
+
+static int enum_max(Port::MetaContainer meta)
+{
+    int max = 0;
+    for(auto m:meta)
+        if(strstr(m.title, "map "))
+            max = atoi(m.title+4);
+
+    for(auto m:meta)
+        if(strstr(m.title, "map "))
+            max = max<atoi(m.title+4) ? atoi(m.title+4) : max;
+
+    return max;
+}
+
+static ostream &add_options(ostream &o, Port::MetaContainer meta)
+{
+    string sym_names = "xyzabcdefghijklmnopqrstuvw";
+    int sym_idx = 0;
+    bool has_options = false;
+    for(auto m:meta)
+        if(strstr(m.title, "map "))
+            has_options = true;
+    for(auto m:meta)
+        if(strcmp(m.title, "documentation") &&
+                strcmp(m.title, "parameter") &&
+                strcmp(m.title, "max") &&
+                strcmp(m.title, "min"))
+        printf("m.title = <%s>\n", m.title);
+
+    if(!has_options)
+        return o;
+
+    o << "    <hints>\n";
+    for(auto m:meta) {
+        if(strstr(m.title, "map ")) {
+            o << "      <point symbol=\"" << sym_names[sym_idx++] << "\" value=\"";
+            o << m.title+4 << "\">" << m.value << "</point>\n";
+        }
+    }
+    o << "    </hints>\n";
+
+    return o;
+}
+static ostream &dump_t_f_port(ostream &o, string name, string doc)
+{
+    o << " <message_in pattern=\"" << name << "\" typetag=\"T\">\n";
+    o << "  <desc>Enable " << doc << "</desc>\n";
+    o << "  <param_T symbol=\"x\"/>\n";
+    o << " </message_in>\n";
+    o << " <message_in pattern=\"" << name << "\" typetag=\"F\">\n";
+    o << "  <desc>Disable "  << doc << "</desc>\n";
+    o << "  <param_F symbol=\"x\"/>\n";
+    o << " </message_in>\n";
+    o << " <message_in pattern=\"" << name << "\" typetag=\"\">\n";
+    o << "  <desc>Get state of " << doc << "</desc>\n";
+    o << " </message_in>\n";
+    o << " <message_out pattern=\"" << name << "\" typetag=\"T\">\n";
+    o << "  <desc>Value of " << doc << "</desc>\n";
+    o << "  <param_T symbol=\"x\"/>";
+    o << " </message_out>\n";
+    o << " <message_out pattern=\"" << name << "\" typetag=\"F\">\n";
+    o << "  <desc>Value of " <<  doc << "</desc>\n";
+    o << "  <param_F symbol=\"x\"/>";
+    o << " </message_out>\n";
+    return o;
+}
+static ostream &dump_any_port(ostream &o, string name, string doc)
+{
+    o << " <message_in pattern=\"" << name << "\" typetag=\"*\">\n";
+    o << "  <desc>" << doc << "</desc>\n";
+    o << " </message_in>\n";
+    return o;
+}
+
+static ostream &dump_generic_port(ostream &o, string name, string doc, string type)
+{
+    const char *t = type.c_str();
+    string arg_names = "xyzabcdefghijklmnopqrstuvw";
+
+    //start out with argument separator
+    if(*t++ != ':')
+        return o;
+    //now real arguments (assume [] don't exist)
+    string args;
+    while(*t && *t != ':')
+        args += *t++;
+
+    o << " <message_in pattern=\"" << name << "\" typetag=\"" << args << "\">\n";
+    o << "  <desc>" << doc << "</desc>\n";
+
+    assert(args.length()<arg_names.length());
+    for(unsigned i=0; i<args.length(); ++i)
+        o << "  <param_" << args[i] << " symbol=\"" << arg_names[i] << "\"/>\n";
+    o << " </message_in>\n";
+
+    if(*t == ':')
+        return dump_generic_port(o, name, doc, t);
+    else
+        return o;
+}
+
 void dump_ports_cb(const rtosc::Port *p, const char *name, void *v)
 {
-    std::ostream &o = *(std::ostream*)v;
-    auto meta = p->meta();
-    if(meta.find("parameter") != p->meta().end()) {
+    std::ostream &o  = *(std::ostream*)v;
+    auto meta        = p->meta();
+    const char *args = strchr(p->name, ':');
+    auto mparameter  = meta.find("parameter");
+    auto mdoc        = meta.find("documentation");
+    string doc;
+
+    if(mdoc != p->meta().end())
+        doc = mdoc.value;
+    if(meta.find("internal") != meta.end()) {
+        doc += "[INTERNAL]";
+    }
+
+    if(mparameter != p->meta().end()) {
         char type = 0;
-        const char *foo = strchr(p->name, ':');
-        if(strchr(foo, 'f'))
-            type = 'f';
-        else if(strchr(foo, 'i'))
-            type = 'i';
-        else if(strchr(foo, 'c'))
-            type = 'c';
-        else if(strchr(foo, 'T'))
-            type = 't';
+        if(args) {
+            if(strchr(args, 'f'))
+                type = 'f';
+            else if(strchr(args, 'i'))
+                type = 'i';
+            else if(strchr(args, 'c'))
+                type = 'c';
+            else if(strchr(args, 'T'))
+                type = 't';
+            else if(strchr(args, 's'))
+                type = 's';
+        }
+
         if(!type) {
-            fprintf(stderr, "rtosc port dumper: Cannot handle '%s'\n", p->name);
+            fprintf(stderr, "rtosc port dumper: Cannot handle '%s'\n", name);
+            fprintf(stderr, "    args = <%s>\n", args);
             return;
         }
 
-        if(type == 't')
-        {
-            o << " <message_in pattern=\"" << name << "\" typetag=\"T\">\n";
-            o << "  <desc>Enable " << p->meta()["documentation"] << "</desc>\n";
-            o << "  <param_T symbol=\"x\"/>\n";
-            o << " </message_in>\n";
-            o << " <message_in pattern=\"" << name << "\" typetag=\"F\">\n";
-            o << "  <desc>Disable "  << p->meta()["documentation"] << "</desc>\n";
-            o << "  <param_F symbol=\"x\"/>\n";
-            o << " </message_in>\n";
-            o << " <message_in pattern=\"" << name << "\" typetag=\"\">\n";
-            o << "  <desc>Get state of " << p->meta()["documentation"] << "</desc>\n";
-            o << " </message_in>\n";
-            o << " <message_out pattern=\"" << name << "\" typetag=\"T\">\n";
-            o << "  <desc>Value of " << p->meta()["documentation"] << "</desc>\n";
-            o << "  <param_T symbol=\"x\"/>";
-            o << " </message_out>\n";
-            o << " <message_out pattern=\"" << name << "\" typetag=\"F\">\n";
-            o << "  <desc>Value of %s</desc>\n", p->meta()["documentation"];
-            o << "  <param_F symbol=\"x\"/>";
-            o << " </message_out>\n";
+        if(type == 't') {
+            dump_t_f_port(o, name, doc);
             return;
         }
 
         o << " <message_in pattern=\"" << name << "\" typetag=\"" << type << "\">\n";
-        o << "  <desc>Set Value of " << p->meta()["documentation"] << "</desc>\n";
-        if(meta.find("min") != meta.end() && meta.find("max") != meta.end() && type != 'c')
-        {
+        o << "  <desc>Set Value of " << doc << "</desc>\n";
+        if(meta.find("min") != meta.end() && meta.find("max") != meta.end() && type != 'c') {
             o << "  <param_" << type << " symbol=\"x\"";
             units(o, meta["unit"]);
             o << ">\n";
             o << "   <range_min_max " << (type == 'f' ? "lmin=\"[\" lmax=\"]\"" : "");
             o << " min=\"" << meta["min"] << "\"  max=\"" << meta["max"] << "\"/>\n";
             o << "  </param_" << type << ">";
+        } else if(meta.find("enumerated") != meta.end()) {
+            o << "  <param_" << type << " symbol=\"x\">\n";
+            o << "    <range_min_max min=\"" << enum_min(meta) << "\" max=\"";
+            o << enum_max(meta) << "\">\n";
+            add_options(o, meta);
+            o << "    </range_min_max>\n";
+            o << "  </param_" << type << ">\n";
         } else {
             o << "  <param_" << type << " symbol=\"x\"";
             units(o, meta["unit"]);
@@ -785,28 +1047,41 @@ void dump_ports_cb(const rtosc::Port *p, const char *name, void *v)
         }
         o << " </message_in>\n";
         o << " <message_in pattern=\"" << name << "\" typetag=\"\">\n";
-        o << "  <desc>Get Value of " << p->meta()["documentation"] << "</desc>\n";
+        o << "  <desc>Get Value of " << doc << "</desc>\n";
         o << " </message_in>\n";
         o << " <message_out pattern=\"" << name << "\" typetag=\"" << type << "\">\n";
-        o << "  <desc>Value of " << p->meta()["documentation"] << "</desc>\n";
-        if(meta.find("min") != meta.end() && meta.find("max") != meta.end() && type != 'c')
-        {
+        o << "  <desc>Value of " << doc << "</desc>\n";
+        if(meta.find("min") != meta.end() && meta.find("max") != meta.end() && type != 'c') {
             o << "  <param_" << type << " symbol=\"x\"";
             units(o, meta["unit"]);
             o << ">\n";
             o << "   <range_min_max " << (type == 'f' ? "lmin=\"[\" lmax=\"]\"" : "");
             o << " min=\"" << meta["min"] << "\"  max=\"" << meta["max"] << "\"/>\n";
             o << "  </param_" << type << ">\n";
+        } else if(meta.find("enumerated") != meta.end()) {
+            o << "  <param_" << type << " symbol=\"x\">\n";
+            o << "    <range_min_max min=\"" << enum_min(meta) << "\" max=\"";
+            o << enum_max(meta) << "\">\n";
+            add_options(o, meta);
+            o << "    </range_min_max>\n";
+            o << "  </param_" << type << ">\n";
         } else {
             o << "  <param_" << type << " symbol=\"x\"";
             units(o, meta["unit"]);
             o << "/>\n";
         }
         o << " </message_out>\n";
-    }// else if(meta.find("documentation") != meta.end())
-    //    fprintf(stderr, "Skipping \"%s\"\n", name);
-    //else
-    //    fprintf(stderr, "Skipping [UNDOCUMENTED] \"%s\"\n", name);
+    } else if(mdoc != meta.end() && (!args || args == std::string(""))) {
+        dump_any_port(o, name, doc);
+    } else if(mdoc != meta.end() && args) {
+        dump_generic_port(o, name, doc, args);
+    } else if(mdoc != meta.end()) {
+        fprintf(stderr, "Skipping \"%s\"\n", name);
+        if(args) {
+            fprintf(stderr, "    type = %s\n", args);
+        }
+    } else
+        fprintf(stderr, "Skipping [UNDOCUMENTED] \"%s\"\n", name);
 }
 
 std::ostream &rtosc::operator<<(std::ostream &o, rtosc::OscDocFormatter &formatter)
