@@ -78,6 +78,58 @@ char rtosc_type(const char *msg, unsigned nargument)
     }
 }
 
+static unsigned arg_start(const char *msg_)
+{
+    const uint8_t *msg = (const uint8_t*)msg_;
+    //Iterate to the right position
+    const uint8_t *args = (const uint8_t*) rtosc_argument_string(msg_);
+    const uint8_t *aligned_ptr = args-1;
+    const uint8_t *arg_pos = args;
+
+    while(*++arg_pos);
+    //Alignment
+    arg_pos += 4-(arg_pos-aligned_ptr)%4;
+    return arg_pos-msg;
+}
+
+static unsigned arg_size(const uint8_t *arg_mem, char type)
+{
+    if(!has_reserved(type))
+        return 0;
+    const uint8_t  *arg_pos=arg_mem;
+    uint32_t blob_length = 0;
+    switch(type)
+    {
+        case 'h':
+        case 't':
+        case 'd':
+            return 8;
+        case 'm':
+        case 'r':
+        case 'f':
+        case 'c':
+        case 'i':
+            return 4;
+        case 'S':
+        case 's':
+            while(*++arg_pos);
+            arg_pos += 4-(arg_pos-arg_mem)%4;
+            return arg_pos-arg_mem;
+        case 'b':
+            blob_length |= (*arg_pos++ << 24);
+            blob_length |= (*arg_pos++ << 16);
+            blob_length |= (*arg_pos++ << 8);
+            blob_length |= (*arg_pos++);
+            if(blob_length%4)
+                blob_length += 4-blob_length%4;
+            arg_pos += blob_length;
+            return arg_pos-arg_mem;
+        default:
+            assert("Invalid Type");
+    }
+    return -1;
+}
+
 static unsigned arg_off(const char *msg, unsigned idx)
 {
     if(!has_reserved(rtosc_type(msg,idx)))
@@ -97,45 +149,11 @@ static unsigned arg_off(const char *msg, unsigned idx)
         ++args;
 
     while(idx--) {
-        uint32_t bundle_length = 0;
-        switch(*args++)
-        {
-            case 'h':
-            case 't':
-            case 'd':
-                arg_pos +=8;
-                break;
-            case 'm':
-            case 'r':
-            case 'f':
-            case 'c':
-            case 'i':
-                arg_pos += 4;
-                break;
-            case 'S':
-            case 's':
-                while(*++arg_pos);
-                arg_pos += 4-(arg_pos-((uint8_t*)aligned_ptr))%4;
-                break;
-            case 'b':
-                bundle_length |= (*arg_pos++ << 24);
-                bundle_length |= (*arg_pos++ << 16);
-                bundle_length |= (*arg_pos++ << 8);
-                bundle_length |= (*arg_pos++);
-                if(bundle_length%4)
-                    bundle_length += 4-bundle_length%4;
-                arg_pos += bundle_length;
-                break;
-            case '[':
-            case ']':
-                //completely ignore array chars
-                ++idx;
-                break;
-            case 'T':
-            case 'F':
-            case 'I':
-                ;
-        }
+        char type = *args++;
+        if(type == '[' || type == ']')
+            idx++;//not a valid arg idx
+        else
+            arg_pos += arg_size(arg_pos, type);
     }
     return arg_pos-(uint8_t*)msg;
 }
@@ -385,10 +403,9 @@ size_t rtosc_amessage(char              *buffer,
     return pos;
 }
 
-rtosc_arg_t rtosc_argument(const char *msg, unsigned idx)
+static rtosc_arg_t extract_arg(const uint8_t *arg_pos, char type)
 {
     rtosc_arg_t result = {0};
-    char type = rtosc_type(msg, idx);
     //trivial case
     if(!has_reserved(type)) {
         switch(type)
@@ -403,7 +420,6 @@ rtosc_arg_t rtosc_argument(const char *msg, unsigned idx)
                 ;
         }
     } else {
-        const unsigned char *arg_pos = (const unsigned char*)msg+arg_off(msg,idx);
         switch(type)
         {
             case 'h':
@@ -450,6 +466,52 @@ rtosc_arg_t rtosc_argument(const char *msg, unsigned idx)
     return result;
 }
 
+static const char *advance_past_dummy_args(const char *args)
+{
+    while(*args == '[' || *args == ']')
+        args++;
+    return args;
+}
+
+rtosc_arg_itr_t rtosc_itr_begin(const char *msg)
+{
+    rtosc_arg_itr_t itr;
+    itr.type_pos  = advance_past_dummy_args(rtosc_argument_string(msg));
+    itr.value_pos = (uint8_t*)(msg+arg_start(msg));
+
+    return itr;
+}
+
+rtosc_arg_val_t rtosc_itr_next(rtosc_arg_itr_t *itr)
+{
+    //current position provides the value
+    rtosc_arg_val_t result = {0,{0}};
+    result.type = *itr->type_pos;
+    if(result.type)
+        result.val = extract_arg(itr->value_pos, result.type);
+
+    //advance
+    itr->type_pos = advance_past_dummy_args(itr->type_pos+1);
+    char type = result.type;
+    int size  = arg_size(itr->value_pos, type);
+    itr->value_pos += size;
+
+
+    return result;
+}
+
+int rtosc_itr_end(rtosc_arg_itr_t itr)
+{
+    return !itr.type_pos  || !*itr.type_pos;
+}
+
+rtosc_arg_t rtosc_argument(const char *msg, unsigned idx)
+{
+    char type = rtosc_type(msg, idx);
+    uint8_t *arg_mem = (uint8_t*)msg + arg_off(msg, idx);
+    return extract_arg(arg_mem, type);
+}
+
 static unsigned char deref(unsigned pos, ring_t *ring)
 {
     return pos<ring[0].len ? ring[0].data[pos] :
@@ -461,10 +523,10 @@ static size_t bundle_ring_length(ring_t *ring)
     unsigned pos = 8+8;//goto first length field
     uint32_t advance = 0;
     do {
-        advance = deref(pos+0, ring) << (8*0) |
-                  deref(pos+1, ring) << (8*1) |
-                  deref(pos+2, ring) << (8*2) |
-                  deref(pos+3, ring) << (8*3);
+        advance = deref(pos+0, ring) << (8*3) |
+                  deref(pos+1, ring) << (8*2) |
+                  deref(pos+2, ring) << (8*1) |
+                  deref(pos+3, ring) << (8*0);
         if(advance)
             pos += 4+advance;
     } while(advance);
@@ -598,6 +660,49 @@ bool rtosc_valid_message_p(const char *msg, size_t len)
     size_t observed_length = rtosc_message_length(msg, len);
     return observed_length == len;
 }
+static uint64_t extract_uint64(const uint8_t *arg_pos)
+{
+    uint64_t arg = 0;
+    arg |= (((uint64_t)*arg_pos++) << 56);
+    arg |= (((uint64_t)*arg_pos++) << 48);
+    arg |= (((uint64_t)*arg_pos++) << 40);
+    arg |= (((uint64_t)*arg_pos++) << 32);
+    arg |= (((uint64_t)*arg_pos++) << 24);
+    arg |= (((uint64_t)*arg_pos++) << 16);
+    arg |= (((uint64_t)*arg_pos++) << 8);
+    arg |= (((uint64_t)*arg_pos++));
+    return arg;
+}
+
+static uint32_t extract_uint32(const uint8_t *arg_pos)
+{
+    uint32_t arg = 0;
+    arg |= (((uint32_t)*arg_pos++) << 24);
+    arg |= (((uint32_t)*arg_pos++) << 16);
+    arg |= (((uint32_t)*arg_pos++) << 8);
+    arg |= (((uint32_t)*arg_pos++));
+    return arg;
+}
+
+static void emplace_uint64(uint8_t *buffer, uint64_t d)
+{
+    buffer[0] = ((d>>56) & 0xff);
+    buffer[1] = ((d>>48) & 0xff);
+    buffer[2] = ((d>>40) & 0xff);
+    buffer[3] = ((d>>32) & 0xff);
+    buffer[4] = ((d>>24) & 0xff);
+    buffer[5] = ((d>>16) & 0xff);
+    buffer[6] = ((d>>8)  & 0xff);
+    buffer[7] = ((d>>0)  & 0xff);
+}
+
+static void emplace_uint32(uint8_t *buffer, uint32_t d)
+{
+    buffer[0] = ((d>>24) & 0xff);
+    buffer[1] = ((d>>16) & 0xff);
+    buffer[2] = ((d>>8)  & 0xff);
+    buffer[3] = ((d>>0)  & 0xff);
+}
 
 size_t rtosc_bundle(char *buffer, size_t len, uint64_t tt, int elms, ...)
 {
@@ -605,15 +710,15 @@ size_t rtosc_bundle(char *buffer, size_t len, uint64_t tt, int elms, ...)
     memset(buffer, 0, len);
     strcpy(buffer, "#bundle");
     buffer += 8;
-    (*(uint64_t*)buffer) = tt;
-    buffer +=8;
+    emplace_uint64((uint8_t*)buffer, tt);
+    buffer += 8;
     va_list va;
     va_start(va, elms);
     for(int i=0; i<elms; ++i) {
         const char   *msg  = va_arg(va, const char*);
         //It is assumed that any passed message/bundle is valid
         size_t        size = rtosc_message_length(msg, -1);
-        *(uint32_t*)buffer = size;
+        emplace_uint32((uint8_t*)buffer, size);
         buffer += 4;
         memcpy(buffer, msg, size);
         buffer+=size;
@@ -623,14 +728,14 @@ size_t rtosc_bundle(char *buffer, size_t len, uint64_t tt, int elms, ...)
     return buffer-_buffer;
 }
 
+
 #define POS ((size_t)(((const char *)lengths) - buffer))
 size_t rtosc_bundle_elements(const char *buffer, size_t len)
 {
     const uint32_t *lengths = (const uint32_t*) (buffer+16);
     size_t elms = 0;
-    //TODO
-    while(POS < len && *lengths) {
-        lengths += *lengths/4+1;
+    while(POS < len && extract_uint32((const uint8_t*)lengths)) {
+        lengths += extract_uint32((const uint8_t*)lengths)/4+1;
 
         if(POS > len)
             break;
@@ -644,7 +749,10 @@ const char *rtosc_bundle_fetch(const char *buffer, unsigned elm)
 {
     const uint32_t *lengths = (const uint32_t*) (buffer+16);
     size_t elm_pos = 0;
-    while(elm_pos!=elm && *lengths) ++elm_pos, lengths+=*lengths/4+1;
+    while(elm_pos!=elm && extract_uint32((const uint8_t*)lengths)) {
+        ++elm_pos;
+        lengths += extract_uint32((const uint8_t*)lengths)/4+1;
+    }
 
     return (const char*) (elm==elm_pos?lengths+1:NULL);
 }
@@ -654,9 +762,9 @@ size_t rtosc_bundle_size(const char *buffer, unsigned elm)
     const uint32_t *lengths = (const uint32_t*) (buffer+16);
     size_t elm_pos = 0;
     size_t last_len = 0;
-    while(elm_pos!=elm && *lengths) {
-        last_len = *lengths;
-        ++elm_pos, lengths+=*lengths/4+1;
+    while(elm_pos!=elm && extract_uint32((const uint8_t*)lengths)) {
+        last_len = extract_uint32((const uint8_t*)lengths);
+        ++elm_pos, lengths+=extract_uint32((const uint8_t*)lengths)/4+1;
     }
 
     return last_len;
@@ -669,5 +777,5 @@ int rtosc_bundle_p(const char *msg)
 
 uint64_t rtosc_bundle_timetag(const char *msg)
 {
-    return *(uint64_t*)(msg+8);
+    return extract_uint64((const uint8_t*)msg+8);
 }
