@@ -26,12 +26,14 @@
 
 #include "EffectMgr.h"
 #include "Effect.h"
+#include "Alienwah.h"
 #include "Reverb.h"
 #include "Echo.h"
 #include "Chorus.h"
 #include "Distorsion.h"
 #include "EQ.h"
 #include "DynamicFilter.h"
+#include "Phaser.h"
 #include "../Misc/XMLwrapper.h"
 #include "../Misc/Util.h"
 #include "../Params/FilterParams.h"
@@ -43,7 +45,8 @@ static const rtosc::Ports local_ports = {
     rSelf(EffectMgr),
     rPaste,
     rRecurp(filterpars, "Filter Parameter for Dynamic Filter"),
-    {"parameter#128::i", rProp(alias) rDoc("Parameter Accessor"), NULL,
+    {"parameter#128::i:T:F", rProp(parameter) rProp(alias) rLinear(0,127) rDoc("Parameter Accessor"),
+        NULL,
         [](const char *msg, rtosc::RtData &d)
         {
             EffectMgr *eff = (EffectMgr*)d.obj;
@@ -52,20 +55,37 @@ static const rtosc::Ports local_ports = {
 
             if(!rtosc_narguments(msg))
                 d.reply(d.loc, "i", eff->geteffectparrt(atoi(mm)));
-            else {
+            else if(rtosc_type(msg, 0) == 'i'){
                 eff->seteffectparrt(atoi(mm), rtosc_argument(msg, 0).i);
                 d.broadcast(d.loc, "i", eff->geteffectparrt(atoi(mm)));
+            } else if(rtosc_type(msg, 0) == 'T'){
+                eff->seteffectparrt(atoi(mm), 127);
+                d.broadcast(d.loc, "i", eff->geteffectparrt(atoi(mm)));
+            } else if(rtosc_type(msg, 0) == 'F'){
+                eff->seteffectparrt(atoi(mm), 0);
+                d.broadcast(d.loc, "i", eff->geteffectparrt(atoi(mm)));
             }
         }},
-    {"preset::i", rProp(alias) rDoc("Effect Preset Selector"), NULL,
+    {"preset::i", rProp(parameter) rProp(alias) rDoc("Effect Preset Selector"), NULL,
         [](const char *msg, rtosc::RtData &d)
         {
+            char loc[1024];
             EffectMgr *eff = (EffectMgr*)d.obj;
             if(!rtosc_narguments(msg))
                 d.reply(d.loc, "i", eff->getpreset());
             else {
                 eff->changepresetrt(rtosc_argument(msg, 0).i);
                 d.broadcast(d.loc, "i", eff->getpreset());
+
+                //update parameters as well
+                strncpy(loc, d.loc, 1024);
+                char *tail = rindex(loc, '/');
+                if(!tail)
+                    return;
+                for(int i=0;i<128;++i) {
+                    sprintf(tail+1, "parameter%d", i);
+                    d.broadcast(loc, "i", eff->geteffectparrt(i));
+                }
             }
         }},
     {"eq-coeffs:", rProp(internal) rDoc("Get equalizer Coefficients"), NULL,
@@ -75,14 +95,15 @@ static const rtosc::Ports local_ports = {
             if(eff->nefx != 7)
                 return;
             EQ *eq = (EQ*)eff->efx;
-            float a[MAX_EQ_BANDS*MAX_FILTER_STAGES*2+1];
-            float b[MAX_EQ_BANDS*MAX_FILTER_STAGES*2+1];
+            float a[MAX_EQ_BANDS*MAX_FILTER_STAGES*3];
+            float b[MAX_EQ_BANDS*MAX_FILTER_STAGES*3];
             memset(a, 0, sizeof(a));
             memset(b, 0, sizeof(b));
             eq->getFilter(a,b);
             d.reply(d.loc, "bb", sizeof(a), a, sizeof(b), b);
         }},
-    {"efftype::i", rDoc("Get Effect Type"), NULL, [](const char *m, rtosc::RtData &d)
+    {"efftype::i", rProp(parameter) rDoc("Get Effect Type"), NULL,
+        [](const char *m, rtosc::RtData &d)
         {
             EffectMgr *eff  = (EffectMgr*)d.obj;
             if(rtosc_narguments(m))  {
@@ -307,8 +328,8 @@ void EffectMgr::out(float *smpsl, float *smpsr)
         return;
     }
     for(int i = 0; i < synth.buffersize; ++i) {
-        smpsl[i]  += denormalkillbuf[i];
-        smpsr[i]  += denormalkillbuf[i];
+        smpsl[i]  += synth.denormalkillbuf[i];
+        smpsr[i]  += synth.denormalkillbuf[i];
         efxoutl[i] = 0.0f;
         efxoutr[i] = 0.0f;
     }
@@ -383,7 +404,7 @@ void EffectMgr::paste(EffectMgr &e)
     changeeffectrt(e.nefx, true);
     changepresetrt(e.preset, true);
     for(int i=0;i<128;++i)
-        seteffectparrt(e.settings[i], i);
+        seteffectparrt(i, e.settings[i]);
 }
 
 void EffectMgr::add2XML(XMLwrapper *xml)
