@@ -46,16 +46,17 @@
 
 using namespace std;
 
-Bank::Bank()
-    :bankpos(0), defaultinsname(" ")
+Bank::Bank(Config *config)
+    :bankpos(0), defaultinsname(" "), config(config),
+     bank_msb(0), bank_lsb(0)
 {
     clearbank();
     bankfiletitle = dirname;
     rescanforbanks();
-    loadbank(config.cfg.currentBankDir);
+    loadbank(config->cfg.currentBankDir);
 
     for(unsigned i=0; i<banks.size(); ++i) {
-        if(banks[i].dir == config.cfg.currentBankDir) {
+        if(banks[i].dir == config->cfg.currentBankDir) {
             bankpos = i;
             break;
         }
@@ -222,6 +223,12 @@ int Bank::loadbank(string bankdirname)
     if(dir == NULL)
         return -1;
 
+    //set msb when possible
+    bank_msb = 0;
+    for(unsigned i=0; i<banks.size(); i++)
+        if(banks[i].dir == bankdirname)
+            bank_msb = i;
+
     dirname = bankdirname;
 
     bankfiletitle = dirname;
@@ -270,7 +277,7 @@ int Bank::loadbank(string bankdirname)
     closedir(dir);
 
     if(!dirname.empty())
-        config.cfg.currentBankDir = dirname;
+        config->cfg.currentBankDir = dirname;
 
     return 0;
 }
@@ -281,7 +288,7 @@ int Bank::loadbank(string bankdirname)
 int Bank::newbank(string newbankdirname)
 {
     string bankdir;
-    bankdir = config.cfg.bankRootDirList[0];
+    bankdir = config->cfg.bankRootDirList[0];
 
     if(((bankdir[bankdir.size() - 1]) != '/')
        && ((bankdir[bankdir.size() - 1]) != '\\'))
@@ -361,8 +368,8 @@ void Bank::rescanforbanks()
     banks.clear();
 
     for(int i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
-        if(!config.cfg.bankRootDirList[i].empty())
-            scanrootdir(config.cfg.bankRootDirList[i]);
+        if(!config->cfg.bankRootDirList[i].empty())
+            scanrootdir(config->cfg.bankRootDirList[i]);
 
     //sort the banks
     sort(banks.begin(), banks.end());
@@ -385,6 +392,18 @@ void Bank::rescanforbanks()
         }
 }
 
+void Bank::setMsb(uint8_t msb)
+{
+    if(msb < banks.size() && banks[msb].dir != bankfiletitle)
+        loadbank(banks[msb].dir);
+}
+
+void Bank::setLsb(uint8_t lsb)
+{
+    //should only involve values of 0/1 for the time being...
+    bank_lsb = limit<uint8_t>(lsb,0,1);
+}
+
 
 // private stuff
 
