@@ -25,6 +25,8 @@
 
 #include "Part.h"
 
+#include "../Misc/Stereo.h"
+#include "../Misc/Util.h"
 #include "../Params/LFOParams.h"
 #include "../Effects/EffectMgr.h"
 #include "../DSP/FFTwrapper.h"
@@ -51,7 +53,9 @@ using namespace rtosc;
 
 static const Ports sysefxPort =
 {
-    {"part#" STRINGIFY(NUM_MIDI_PARTS) "::i", 0, 0, [](const char *m, RtData&d)
+    {"part#" STRINGIFY(NUM_MIDI_PARTS) "::i", rProp(parameter)
+        rDoc("gain on part to sysefx routing"), 0,
+        [](const char *m, RtData&d)
         {
             //ok, this is going to be an ugly workaround
             //we know that if we are here the message previously MUST have
@@ -69,16 +73,18 @@ static const Ports sysefxPort =
             int ind2 = atoi(m);
             Master &mast = *(Master*)d.obj;
 
-            if(rtosc_narguments(m))
+            if(rtosc_narguments(m)) {
                 mast.setPsysefxvol(ind2, ind1, rtosc_argument(m,0).i);
-            else
-                d.reply(d.loc, "i", mast.Psysefxvol[ind2][ind1]);
+                d.broadcast(d.loc, "i", mast.Psysefxvol[ind1][ind2]);
+            } else
+                d.reply(d.loc, "i", mast.Psysefxvol[ind1][ind2]);
         }}
 };
 
 static const Ports sysefsendto =
 {
-    {"to#" STRINGIFY(NUM_SYS_EFX) "::i", 0, 0, [](const char *m, RtData&d)
+    {"to#" STRINGIFY(NUM_SYS_EFX) "::i", 
+        rProp(parameter) rDoc("sysefx to sysefx routing gain"), 0, [](const char *m, RtData&d)
         {
             //same ugly workaround as before
             const char *index_1 = m;
@@ -101,19 +107,25 @@ static const Ports sysefsendto =
 };
 
 static const Ports master_ports = {
+    rString(last_xmz, XMZ_PATH_MAX, "File name for last name loaded if any."),
     rRecursp(part, 16, "Part"),//NUM_MIDI_PARTS
     rRecursp(sysefx, 4, "System Effect"),//NUM_SYS_EFX
     rRecursp(insefx, 8, "Insertion Effect"),//NUM_INS_EFX
     rRecur(microtonal, "Micrtonal Mapping Functionality"),
     rRecur(ctl, "Controller"),
-    rParamZyn(Pkeyshift,  "Global Key Shift"),
     rArrayI(Pinsparts, NUM_INS_EFX, "Part to insert part onto"),
+    {"Pkeyshift::i", rProp(parameter) rLinear(0,127) rDoc("Global Key Shift"), 0, [](const char *m, RtData&d) {
+        if(rtosc_narguments(m)==0) {
+            d.reply(d.loc, "i", ((Master*)d.obj)->Pkeyshift);
+        } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
+            ((Master*)d.obj)->setPkeyshift(limit<char>(rtosc_argument(m,0).i,0,127));
+            d.broadcast(d.loc, "i", ((Master*)d.obj)->Pkeyshift);}}},
     {"echo", rDoc("Hidden port to echo messages"), 0, [](const char *m, RtData&d) {
        d.reply(m-1);}},
-    {"get-vu", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
+    {"get-vu:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
        d.reply("/vu-meter", "bb", sizeof(m->vu), &m->vu, sizeof(float)*NUM_MIDI_PARTS, m->vuoutpeakpart);}},
-    {"reset-vu", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
+    {"reset-vu:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
        m->vuresetpeaks();}},
     {"load-part:ib", rProp(internal) rDoc("Load Part From Middleware"), 0, [](const char *msg, RtData &d) {
@@ -125,25 +137,20 @@ static const Ports master_ports = {
        d.reply("/free", "sb", "Part", sizeof(void*), &m->part[i]);
        m->part[i] = p;
        p->initialize_rt();
-       //printf("part %d is now pointer %p\n", i, p);
-                                                                                                          }},
-    {"Pvolume::i", rDoc("Master Volume"), 0,
+       }},
+    {"Pvolume::i", rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
             ((Master*)d.obj)->setPvolume(limit<char>(rtosc_argument(m,0).i,0,127));
             d.broadcast(d.loc, "i", ((Master*)d.obj)->Pvolume);}}},
-    {"volume::i", rDoc("Master Volume"), 0,
+    {"volume::i", rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
-            //printf("looking at value %d\n", rtosc_argument(m,0).i);
-            //printf("limited value is %d\n", limit<char>(
-            //            rtosc_argument(m,0).i, 0,127));
             ((Master*)d.obj)->setPvolume(limit<char>(rtosc_argument(m,0).i,0,127));
-            //printf("sets volume to value %d\n", ((Master*)d.obj)->Pvolume);
             d.broadcast(d.loc, "i", ((Master*)d.obj)->Pvolume);}}},
     {"Psysefxvol#" STRINGIFY(NUM_SYS_EFX) "/::i", 0, &sysefxPort,
         [](const char *msg, rtosc::RtData &d) {
@@ -165,40 +172,42 @@ static const Ports master_ports = {
         [](const char *m,RtData &d){
             Master *M =  (Master*)d.obj;
             M->noteOff(rtosc_argument(m,0).i,rtosc_argument(m,1).i);}},
+    {"virtual_midi_cc:iii", rDoc("MIDI CC Event"), 0,
+        [](const char *m,RtData &d){
+            Master *M =  (Master*)d.obj;
+            M->setController(rtosc_argument(m,0).i,rtosc_argument(m,1).i,rtosc_argument(m,2).i);}},
 
     {"setController:iii", rDoc("MIDI CC Event"), 0,
         [](const char *m,RtData &d){
             Master *M =  (Master*)d.obj;
             M->setController(rtosc_argument(m,0).i,rtosc_argument(m,1).i,rtosc_argument(m,2).i);}},
-    {"Panic:", rDoc("Stop All Sound"), 0,
+    {"Panic:", rDoc("Stop all sound"), 0,
         [](const char *, RtData &d) {
             Master &M =  *(Master*)d.obj;
             M.ShutUp();
         }},
-    {"freeze_state:", rDoc("Internal Read-only Mode"), 0,
+    {"freeze_state:", rProp(internal) rDoc("Disable OSC event handling\n"
+            "This sets up a read-only mode from which it's safe for another"
+            " thread to save parameters"), 0,
         [](const char *,RtData &d) {
             Master *M =  (Master*)d.obj;
             std::atomic_thread_fence(std::memory_order_release);
             M->frozenState = true;
             d.reply("/state_frozen", "");}},
-    {"thaw_state:", rDoc("Internal Read-only Mode"), 0,
+    {"thaw_state:", rProp(internal) rDoc("Resume handling OSC messages\n"
+            "See /freeze_state for more information"), 0,
         [](const char *,RtData &d) {
             Master *M =  (Master*)d.obj;
             M->frozenState = false;}},
-    {"register:iis", rDoc("MIDI Mapping Registration"), 0,
-        [](const char *m,RtData &d){
+    {"midi-learn/", 0, &rtosc::MidiMapperRT::ports,
+        [](const char *msg, RtData &d) {
             Master *M =  (Master*)d.obj;
-            M->midi.addElm(rtosc_argument(m,0).i, rtosc_argument(m,1).i,rtosc_argument(m,2).s);}},
-    {"learn:s", rDoc("Begin Learning for specified address"), 0,
-        [](const char *m, RtData &d){
-            Master *M =  (Master*)d.obj;
-            printf("learning '%s'\n", rtosc_argument(m,0).s);
-            M->midi.learn(rtosc_argument(m,0).s);}},
-    {"unlearn:s", rDoc("Remove Learning for specified address"), 0,
-        [](const char *m, RtData &d){
-            Master *M =  (Master*)d.obj;
-            M->midi.clear_entry(rtosc_argument(m,0).s);}},
-    {"close-ui", rDoc("Request to close any connection named \"GUI\""), 0, [](const char *, RtData &d) {
+            SNIP;
+            printf("residue message = <%s>\n", msg);
+            d.obj = &M->midi;
+            rtosc::MidiMapperRT::ports.dispatch(msg,d);}},
+    {"close-ui:", rDoc("Request to close any connection named \"GUI\""), 0,
+        [](const char *, RtData &d) {
        d.reply("/close-ui", "");}},
     {"add-rt-memory:bi", rProp(internal) rDoc("Add Additional Memory To RT MemPool"), 0,
         [](const char *msg, RtData &d)
@@ -209,33 +218,28 @@ static const Ports master_ports = {
             m.memory->addMemory(mem, i);
             m.pendingMemory = false;
         }},
-    {"samplerate:", rMap(unit, Hz) rDoc("Synthesizer Global Sample Rate"), 0, [](const char *, RtData &d) {
+    {"samplerate:", rMap(unit, Hz) rDoc("Get synthesizer sample rate"), 0, [](const char *, RtData &d) {
             Master &m = *(Master*)d.obj;
             d.reply("/samplerate", "f", m.synth.samplerate_f);
         }},
-    {"oscilsize:", rDoc("Synthesizer Global Oscillator Size"), 0, [](const char *, RtData &d) {
+    {"oscilsize:", rDoc("Get synthesizer oscillator size"), 0, [](const char *, RtData &d) {
             Master &m = *(Master*)d.obj;
             d.reply("/oscilsize", "f", m.synth.oscilsize_f);
             d.reply("/oscilsize", "i", m.synth.oscilsize);
         }},
-    {"undo_pause",0,0,[](const char *, rtosc::RtData &d)
-        {d.reply("/undo_pause", "");}},
-    {"undo_resume",0,0,[](const char *, rtosc::RtData &d)
-        {d.reply("/undo_resume", "");}},
-    {"config/", 0, &Config::ports, [](const char *, rtosc::RtData &){}},
-    {"presets/", 0, &preset_ports, rBOIL_BEGIN
+    {"undo_pause:",rProp(internal) rDoc("pause undo event recording"),0,
+        [](const char *, rtosc::RtData &d) {d.reply("/undo_pause", "");}},
+    {"undo_resume:",rProp(internal) rDoc("resume undo event recording"),0,
+        [](const char *, rtosc::RtData &d) {d.reply("/undo_resume", "");}},
+    {"config/", rDoc("Top Level Application Configuration Parameters"), &Config::ports,
+        [](const char *, rtosc::RtData &d){d.forward();}},
+    {"presets/", rDoc("Parameter Presets"), &preset_ports, rBOIL_BEGIN
         SNIP
             preset_ports.dispatch(msg, data);
         rBOIL_END},
 };
 const Ports &Master::ports = master_ports;
 
-#ifndef PLUGINVERSION
-//XXX HACKS
-Master *the_master;
-rtosc::ThreadLink *the_bToU;
-#endif
-
 class DataObj:public rtosc::RtData
 {
     public:
@@ -246,6 +250,7 @@ class DataObj:public rtosc::RtData
             loc_size = loc_size_;
             obj      = obj_;
             bToU     = bToU_;
+            forwarded = false;
         }
 
         virtual void reply(const char *path, const char *args, ...) override
@@ -274,9 +279,18 @@ class DataObj:public rtosc::RtData
         }
         virtual void broadcast(const char *msg) override
         {
-            reply("/broadcast");
+            reply("/broadcast", "");
             reply(msg);
-        };
+        }
+
+        virtual void forward(const char *reason) override
+        {
+            assert(message);
+            reply("/forward", "");
+            printf("forwarding '%s'\n", message);
+            forwarded = true;
+        }
+        bool forwarded;
     private:
         rtosc::ThreadLink *bToU;
 };
@@ -286,21 +300,27 @@ vuData::vuData(void)
       rmspeakl(0.0f), rmspeakr(0.0f), clipped(0)
 {}
 
-Master::Master(const SYNTH_T &synth_)
-:HDDRecorder(synth_), ctl(synth_), midi(Master::ports), frozenState(false), pendingMemory(false), synth(synth_)
+Master::Master(const SYNTH_T &synth_, Config* config)
+    :HDDRecorder(synth_), ctl(synth_),
+    microtonal(config->cfg.GzipCompression), bank(config),
+    frozenState(false), pendingMemory(false),
+    synth(synth_), time(synth), gzip_compression(config->cfg.GzipCompression)
 {
     bToU = NULL;
     uToB = NULL;
-    memory = new Allocator();
+
+    //Setup MIDI
+    midi.frontend = [this](const char *msg) {bToU->raw_write(msg);};
+    midi.backend  = [this](const char *msg) {applyOscEvent(msg);};
+
+    memory = new AllocatorClass();
     swaplr = 0;
     off  = 0;
     smps = 0;
     bufl = new float[synth.buffersize];
     bufr = new float[synth.buffersize];
-#ifndef PLUGINVERSION
-    the_master = this;
-#endif
 
+    last_xmz[0] = 0;
     fft = new FFTwrapper(synth.oscilsize);
 
     shutup = 0;
@@ -310,7 +330,8 @@ Master::Master(const SYNTH_T &synth_)
     }
 
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
-        part[npart] = new Part(*memory, synth, &microtonal, fft);
+        part[npart] = new Part(*memory, synth, time, config->cfg.GzipCompression,
+                               config->cfg.Interpolation, &microtonal, fft);
 
     //Insertion Effects init
     for(int nefx = 0; nefx < NUM_INS_EFX; ++nefx)
@@ -323,24 +344,6 @@ Master::Master(const SYNTH_T &synth_)
 
     defaults();
 
-#ifndef PLUGINVERSION
-    midi.event_cb = [](const char *m)
-    {
-        char loc_buf[1024];
-        DataObj d{loc_buf, 1024, the_master, the_bToU};
-        memset(loc_buf, 0, sizeof(loc_buf));
-        //printf("sending an event to the owner of '%s'\n", m);
-        Master::ports.dispatch(m+1, d);
-    };
-#else
-    midi.event_cb = [](const char *) {};
-#endif
-
-    midi.error_cb = [](const char *a, const char *b)
-    {
-        fprintf(stderr, "MIDI- got an error '%s' -- '%s'\n",a,b);
-    };
-
     mastercb = 0;
     mastercb_ptr = 0;
 }
@@ -351,9 +354,19 @@ void Master::applyOscEvent(const char *msg)
     DataObj d{loc_buf, 1024, this, bToU};
     memset(loc_buf, 0, sizeof(loc_buf));
     d.matches = 0;
-    ports.dispatch(msg+1, d);
-    if(d.matches == 0)
-        fprintf(stderr, "Unknown path '%s'\n", msg);
+        
+    if(strcmp(msg, "/get-vu") && false) {
+        fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 5 + 30, 0 + 40);
+        fprintf(stdout, "backend[*]: '%s'<%s>\n", msg,
+                rtosc_argument_string(msg));
+        fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
+    }
+
+    ports.dispatch(msg, d, true);
+    if(d.matches == 0 && !d.forwarded)
+        fprintf(stderr, "Unknown path '%s:%s'\n", msg, rtosc_argument_string(msg));
+    if(d.forwarded)
+        bToU->raw_write(msg);
 }
 
 void Master::defaults()
@@ -439,13 +452,14 @@ void Master::setController(char chan, int type, int par)
 {
     if(frozenState)
         return;
-    midi.process(chan,type,par);
+    //TODO add chan back
+    midi.handleCC(type,par);
     if((type == C_dataentryhi) || (type == C_dataentrylo)
        || (type == C_nrpnhi) || (type == C_nrpnlo)) { //Process RPN and NRPN by the Master (ignore the chan)
         ctl.setparameternumber(type, par);
 
         int parhi = -1, parlo = -1, valhi = -1, vallo = -1;
-        if(ctl.getnrpn(&parhi, &parlo, &valhi, &vallo) == 0) //this is NRPN
+        if(ctl.getnrpn(&parhi, &parlo, &valhi, &vallo) == 0) { //this is NRPN
             switch(parhi) {
                 case 0x04: //System Effects
                     if(parlo < NUM_SYS_EFX)
@@ -456,14 +470,8 @@ void Master::setController(char chan, int type, int par)
                         insefx[parlo]->seteffectparrt(valhi, vallo);
                     break;
             }
-    }
-    else
-    if(type == C_bankselectmsb) {      // Change current bank
-        //if(((unsigned int)par < bank.banks.size())
-        //   && (bank.banks[par].dir != bank.bankfiletitle))
-        //    bank.loadbank(bank.banks[par].dir);
-    }
-    else {  //other controllers
+        }
+    } else {  //other controllers
         for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) //Send the controller to all part assigned to the channel
             if((chan == part[npart]->Prcvchn) && (part[npart]->Penabled != 0))
                 part[npart]->SetController(type, par);
@@ -626,7 +634,7 @@ void Master::AudioOut(float *outl, float *outr)
     DataObj d{loc_buf, 1024, this, bToU};
     memset(loc_buf, 0, sizeof(loc_buf));
     int events = 0;
-    while(uToB && uToB->hasNext() && events < 10) {
+    while(uToB && uToB->hasNext() && events < 100) {
         const char *msg = uToB->read();
 
         if(!strcmp(msg, "/load-master")) {
@@ -647,9 +655,7 @@ void Master::AudioOut(float *outl, float *outr)
                     rtosc_argument_string(msg));
             fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
         }
-        d.matches = 0;
-        //fprintf(stdout, "address '%s'\n", uToB->peak());
-        ports.dispatch(msg+1, d);
+        ports.dispatch(msg, d, true);
         events++;
         if(!d.matches) {// && !ports.apropos(msg)) {
             fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 1, 7 + 30, 0 + 40);
@@ -813,8 +819,8 @@ void Master::AudioOut(float *outl, float *outr)
         ShutUp();
     }
 
-    //update the LFO's time
-    LFOParams::time++;
+    //update the global frame timer
+    time++;
 }
 
 //TODO review the respective code from yoshimi for this
@@ -1044,13 +1050,12 @@ int Master::saveXML(const char *filename)
     add2XML(xml);
     xml->endbranch();
 
-    int result = xml->saveXMLfile(filename);
+    int result = xml->saveXMLfile(filename, gzip_compression);
     delete (xml);
     return result;
 }
 
 
-
 int Master::loadXML(const char *filename)
 {
     XMLwrapper *xml = new XMLwrapper();
