@@ -28,6 +28,7 @@
 #include <rtosc/miditable.h>
 #include <rtosc/ports.h>
 
+#include "Time.h"
 #include "Bank.h"
 #include "Recorder.h"
 
@@ -49,10 +50,12 @@ class Master
 {
     public:
         /** Constructor TODO make private*/
-        Master(const SYNTH_T &synth);
+        Master(const SYNTH_T &synth, class Config *config);
         /** Destructor*/
         ~Master();
 
+        char last_xmz[XMZ_PATH_MAX];
+
         void applyOscEvent(const char *event);
 
         /**Saves all settings to a XML file
@@ -64,7 +67,6 @@ class Master
 
         void defaults();
 
-
         /**loads all settings from a XML file
          * @return 0 for ok or -1 if there is an error*/
         int loadXML(const char *filename);
@@ -99,7 +101,8 @@ class Master
 
         /**Audio Output*/
         void AudioOut(float *outl, float *outr) REALTIME;
-        /**Audio Output (for callback mode). This allows the program to be controled by an external program*/
+        /**Audio Output (for callback mode).
+         * This allows the program to be controled by an external program*/
         void GetAudioOutSamples(size_t nsamples,
                                 unsigned samplerate,
                                 float *outl,
@@ -163,7 +166,7 @@ class Master
         //Statistics on output levels
         vuData vu;
 
-        rtosc::MidiTable midi;//<1024,64>
+        rtosc::MidiMapperRT midi;
 
         bool   frozenState;//read-only parameters for threadsafe actions
         Allocator *memory;
@@ -171,6 +174,8 @@ class Master
         rtosc::ThreadLink *uToB;
         bool pendingMemory;
         const SYNTH_T &synth;
+        AbsTime  time;
+        const int& gzip_compression; //!< value from config
     private:
         float  sysefxvol[NUM_SYS_EFX][NUM_MIDI_PARTS];
         float  sysefxsend[NUM_SYS_EFX][NUM_SYS_EFX];
