@@ -58,16 +58,16 @@ const rtosc::Ports Microtonal::ports = {
     rParamZyn(Pmiddlenote, "Scale degree 0 note"),
 
     //TODO check to see if this should be exposed
-    rParamZyn(Pmapsize, "UNDOCUMENTED"),
+    rParamZyn(Pmapsize, "Size of key map"),
     rToggle(Pmappingenabled, "Mapping Enable"),
 
-    rParams(Pmapping, "UNDOCUMENTED"),
+    rParams(Pmapping, 128, "Mapping of keys"),
     rParamZyn(Pglobalfinedetune, "Fine detune for all notes"),
 
     rString(Pname, MICROTONAL_MAX_NAME_LEN, "Microtonal Name"),
     rString(Pcomment, MICROTONAL_MAX_NAME_LEN, "Microtonal Name"),
 
-    {"octavesize:", 0, 0, [](const char*, RtData &d)
+    {"octavesize:", rDoc("Get octave size"), 0, [](const char*, RtData &d)
         {
             Microtonal &m = *(Microtonal*)d.obj;
             d.reply(d.loc, "i", m.getoctavesize());
@@ -75,7 +75,8 @@ const rtosc::Ports Microtonal::ports = {
 };
 
 
-Microtonal::Microtonal()
+Microtonal::Microtonal(const int &gzip_compression)
+    : gzip_compression(gzip_compression)
 {
     defaults();
 }
@@ -654,7 +655,7 @@ void Microtonal::getfromXML(XMLwrapper *xml)
                     //populate fields for display
                     float x = logf(octave[i].tuning) / LOG_2 * 1200.0f;
                     octave[i].x1 = (int) floor(x);
-                    octave[i].x2 = (int) (floor(fmodf(x, 1.0f) * 1e6));
+                    octave[i].x2 = (int) (floor((x-octave[i].x1) * 1.0e6));
                 }
 
 
@@ -688,7 +689,7 @@ int Microtonal::saveXML(const char *filename) const
     add2XML(xml);
     xml->endbranch();
 
-    int result = xml->saveXMLfile(filename);
+    int result = xml->saveXMLfile(filename, gzip_compression);
     delete (xml);
     return result;
 }
