@@ -31,7 +31,7 @@
 #include <iostream>
 #include <sstream>
 
-#include "../globals.h"
+#include "globals.h"
 #include "Util.h"
 
 using namespace std;
@@ -98,7 +98,7 @@ XMLwrapper::XMLwrapper()
 {
     version.Major    = 2;
     version.Minor    = 5;
-    version.Revision = 1;
+    version.Revision = 2;
 
     minimal = true;
 
@@ -187,13 +187,12 @@ bool XMLwrapper::hasPadSynth() const
 
 /* SAVE XML members */
 
-int XMLwrapper::saveXMLfile(const string &filename) const
+int XMLwrapper::saveXMLfile(const string &filename, int compression) const
 {
     char *xmldata = getXMLdata();
     if(xmldata == NULL)
         return -2;
 
-    int compression = config.cfg.GzipCompression;
     int result      = dosavefile(filename.c_str(), compression, xmldata);
 
     free(xmldata);
@@ -251,8 +250,12 @@ void XMLwrapper::addpar(const string &name, int val)
 
 void XMLwrapper::addparreal(const string &name, float val)
 {
-    addparams("par_real", 2, "name", name.c_str(), "value",
-              stringFrom<float>(val).c_str());
+    union { float in; uint32_t out; } convert;
+    char buf[11];
+    convert.in = val;
+    sprintf(buf, "0x%8X", convert.out);
+    addparams("par_real", 3, "name", name.c_str(), "value",
+              stringFrom<float>(val).c_str(), "exact_value", buf);
 }
 
 void XMLwrapper::addparbool(const string &name, int val)
@@ -572,7 +575,14 @@ float XMLwrapper::getparreal(const char *name, float defaultpar) const
     if(tmp == NULL)
         return defaultpar;
 
-    const char *strval = mxmlElementGetAttr(tmp, "value");
+    const char *strval = mxmlElementGetAttr(tmp, "exact_value");
+    if (strval != NULL) {
+        union { float out; uint32_t in; } convert;
+        sscanf(strval+2, "%x", &convert.in);
+        return convert.out;
+    }
+
+    strval = mxmlElementGetAttr(tmp, "value");
     if(strval == NULL)
         return defaultpar;
 
@@ -622,3 +632,55 @@ mxml_node_t *XMLwrapper::addparams(const char *name, unsigned int params,
     }
     return element;
 }
+
+XmlNode::XmlNode(std::string name_)
+    :name(name_)
+{}
+
+std::string &XmlNode::operator[](std::string name)
+{
+    //fetch an existing one
+    for(auto &a:attrs)
+        if(a.name == name)
+            return a.value;
+
+    //create a new one
+    attrs.push_back({name, ""});
+    return attrs[attrs.size()-1].value;
+}
+
+bool XmlNode::has(std::string name_)
+{
+    //fetch an existing one
+    for(auto &a:attrs)
+        if(a.name == name_)
+            return true;
+    return false;
+}
+
+void XMLwrapper::add(const XmlNode &node_)
+{
+    mxml_node_t *element = mxmlNewElement(node, node_.name.c_str());
+    for(auto attr:node_.attrs)
+        mxmlElementSetAttr(element, attr.name.c_str(),
+                attr.value.c_str());
+}
+
+std::vector<XmlNode> XMLwrapper::getBranch(void) const
+{
+    std::vector<XmlNode> res;
+    mxml_node_t *current = node->child;
+    while(current) {
+        if(current->type == MXML_ELEMENT) {
+            auto elm = current->value.element;
+            XmlNode n(elm.name);
+            for(int i=0; i<elm.num_attrs; ++i) {
+                auto &attr = elm.attrs[i];
+                n[attr.name] = attr.value;
+            }
+            res.push_back(n);
+        }
+        current = mxmlWalkNext(current, node, MXML_NO_DESCEND);
+    }
+    return res;
+}
