@@ -14,7 +14,9 @@ struct SYNTH_T;
 class EngineMgr
 {
     public:
-        static EngineMgr &getInstance(const SYNTH_T *synth=NULL);
+        static EngineMgr &getInstance(
+            const SYNTH_T *synth = nullptr,
+            const class oss_devs_t* oss_devs = nullptr);
         ~EngineMgr();
 
         /**Gets requested engine
@@ -39,6 +41,6 @@ class EngineMgr
         AudioOut *defaultOut;
         MidiIn   *defaultIn;
     private:
-        EngineMgr(const SYNTH_T *synth);
+        EngineMgr(const SYNTH_T *synth, const oss_devs_t &oss_devs);
 };
 #endif
