@@ -29,7 +29,6 @@
 
 #include "DSSIaudiooutput.h"
 #include "../Misc/Master.h"
-#include "../Misc/Config.h"
 #include "../Misc/Bank.h"
 #include "../Misc/Util.h"
 #include <rtosc/thread-link.h>
@@ -371,6 +370,7 @@ const DSSI_Program_Descriptor *DSSIaudiooutput::getProgram(unsigned long index)
  */
 void DSSIaudiooutput::selectProgram(unsigned long bank, unsigned long program)
 {
+    middleware->pendingSetBank(bank);
     middleware->pendingSetProgram(0, program);
 }
 
@@ -534,7 +534,7 @@ DSSI_Descriptor *DSSIaudiooutput::initDssiDescriptor()
             newLadspaDescriptor->Name  = "ZynAddSubFX";
             newLadspaDescriptor->Maker =
                 "Nasca Octavian Paul <zynaddsubfx@yahoo.com>";
-            newLadspaDescriptor->Copyright = "GNU General Public License v2";
+            newLadspaDescriptor->Copyright = "GNU General Public License v2 or later";
             newLadspaDescriptor->PortCount = 2;
 
             newPortNames    = new const char *[newLadspaDescriptor->PortCount];
@@ -611,12 +611,9 @@ DSSIaudiooutput::DSSIaudiooutput(unsigned long sampleRate)
     config.init();
 
     sprng(time(NULL));
-    denormalkillbuf = new float [synth.buffersize];
-    for(int i = 0; i < synth.buffersize; i++)
-        denormalkillbuf[i] = (RND - 0.5f) * 1e-16;
 
     synth.alias();
-    middleware = new MiddleWare(synth);
+    middleware = new MiddleWare(std::move(synth), &config);
     initBanks();
     loadThread = new std::thread([this]() {
             while(middleware) {
@@ -649,20 +646,6 @@ void DSSIaudiooutput::initBanks(void)
     }
 }
 
-/**
- * constructor for the internally used ProgramDescriptor class
- *
- * @param _bank [in] bank number
- * @param _program [in] program number
- * @param _name [in] instrument / sample name
- * @return
- */
-DSSIaudiooutput::ProgramDescriptor::ProgramDescriptor(unsigned long _bank,
-                                                      unsigned long _program,
-                                                      char *_name)
-    :bank(_bank), program(_program), name(_name)
-{}
-
 /**
  * The map of programs available; held as a single shared statically allocated object.
  */
@@ -683,23 +666,21 @@ long DSSIaudiooutput::bankNoToMap = 1;
  */
 bool DSSIaudiooutput::mapNextBank()
 {
-    Bank &bank = middleware->spawnMaster()->bank;
-    bool  retval;
-    if((bankNoToMap >= (int)bank.banks.size())
-       || bank.banks[bankNoToMap].dir.empty())
-        retval = false;
+    Bank &bank  = middleware->spawnMaster()->bank;
+    auto &banks = bank.banks;
+    if(bankNoToMap >= (int)banks.size() || banks[bankNoToMap].dir.empty())
+        return false;
     else {
         bank.loadbank(bank.banks[bankNoToMap].dir);
-        for(unsigned long instrument = 0; instrument < BANK_SIZE;
-            ++instrument) {
+        for(int instrument = 0; instrument < BANK_SIZE; ++instrument) {
             string insName = bank.getname(instrument);
-            if(!insName.empty() && (insName[0] != '\0') && (insName[0] != ' '))
-                programMap.push_back(ProgramDescriptor(bankNoToMap, instrument,
-                                                       const_cast<char *>(
-                                                           insName.c_str())));
+            ProgramDescriptor prog{(unsigned long)bankNoToMap,
+                                   (unsigned long)instrument, insName};
+
+            if(!insName.empty() && insName[0] != '\0' && insName[0] != ' ')
+                programMap.push_back(prog);
         }
         bankNoToMap++;
-        retval = true;
+        return true;
     }
-    return retval;
 }
