@@ -22,6 +22,7 @@
 
 #include <cmath>
 #include <cstdlib>
+#include <cassert>
 #include <rtosc/ports.h>
 #include <rtosc/port-sugar.h>
 
@@ -126,13 +127,30 @@ EnvelopeParams::EnvelopeParams(unsigned char Penvstretch_,
 EnvelopeParams::~EnvelopeParams()
 {}
 
+#define COPY(y) this->y = ep.y
 void EnvelopeParams::paste(const EnvelopeParams &ep)
 {
-    //Avoid undefined behavior
-    if(&ep == this)
-        return;
-    memcpy((char*)this, (const char*)&ep, sizeof(*this));
+
+    COPY(Pfreemode);
+    COPY(Penvpoints);
+    COPY(Penvsustain);
+    for(int i=0; i<MAX_ENVELOPE_POINTS; ++i) {
+        this->Penvdt[i]  = ep.Penvdt[i];
+        this->Penvval[i] = ep.Penvval[i];
+    }
+    COPY(Penvstretch);
+    COPY(Pforcedrelease);
+    COPY(Plinearenvelope);
+
+    COPY(PA_dt);
+    COPY(PD_dt);
+    COPY(PR_dt);
+    COPY(PA_val);
+    COPY(PD_val);
+    COPY(PS_val);
+    COPY(PR_val);
 }
+#undef COPY
 
 float EnvelopeParams::getdt(char i) const
 {
