@@ -34,79 +34,155 @@
 using namespace rtosc;
 
 
-#define PC(x) rParamZyn(P##x, "undocumented padnote parameter")
-
-template<int i>
-void simpleset(const char *m, rtosc::RtData &d)
-{
-    unsigned char *addr = ((unsigned char*) d.obj)+i;
-    if(!rtosc_narguments(m))
-        d.reply(d.loc, "c", *addr);
-    else
-        *addr = rtosc_argument(m, 0).i;
-}
-
 #define rObject PADnoteParameters
-
-#define P_C(x) rtosc::Port{#x "::c", "::", NULL, \
-    simpleset<__builtin_offsetof(class PADnoteParameters, P##x)>}
-static const rtosc::Ports PADnotePorts =
+static const rtosc::Ports realtime_ports = 
 {
-    rRecurp(oscilgen, "Oscillator"),
     rRecurp(FreqLfo, "Frequency LFO"),
     rRecurp(AmpLfo,   "Amplitude LFO"),
     rRecurp(FilterLfo, "Filter LFO"),
-    rRecurp(resonance, "Resonance"),
     rRecurp(FreqEnvelope, "Frequency Envelope"),
     rRecurp(AmpEnvelope, "Amplitude Envelope"),
     rRecurp(FilterEnvelope, "Filter Envelope"),
     rRecurp(GlobalFilter, "Post Filter"),
-    rParamI(Pmode, rMap(min, 0), rMap(max, 2), "0 - bandwidth, 1 - discrete 2 - continious"),
-    PC(Volume),
-    PC(hp.base.type),
-    PC(hp.base.par1),
-    PC(hp.freqmult),
-    PC(hp.modulator.par1),
-    PC(hp.modulator.freq),
-    PC(hp.width),
-    PC(hp.amp.mode),
-    PC(hp.amp.type),
-    PC(hp.amp.par1),
-    PC(hp.amp.par2),
-    rToggle(Php.autoscale, "Autoscaling Harmonics"),
-    PC(hp.onehalf),
 
-    PC(bwscale),
+    //Volume
+    rToggle(PStereo, "Stereo/Mono Mode"),
+    rParamZyn(PPanning, "Left Right Panning"),
+    rParamZyn(PVolume, "Synth Volume"),
+    rParamZyn(PAmpVelocityScaleFunction, "Amplitude Velocity Sensing function"),
 
-    PC(hrpos.type),
-    PC(hrpos.par1),
-    PC(hrpos.par2),
-    PC(hrpos.par3),
+    rParamZyn(Fadein_adjustment, "Adjustment for anti-pop strategy."),
 
-    PC(quality.samplesize),
-    PC(quality.basenote),
-    PC(quality.oct),
-    PC(quality.smpoct),
+    //Punch
+    rParamZyn(PPunchStrength, "Punch Strength"),
+    rParamZyn(PPunchTime, "UNKNOWN"),
+    rParamZyn(PPunchStretch, "How Punch changes with note frequency"),
+    rParamZyn(PPunchVelocitySensing, "Punch Velocity control"),
 
-    PC(fixedfreq),
-    PC(fixedfreqET),
-    //TODO detune, coarse detune
-    PC(DetuneType),
-    PC(Stereo),
-    PC(Panning),
-    PC(AmpVelocityScaleFunction),
-    PC(PunchStrength),
-    PC(PunchTime),
-    PC(PunchStretch),
-    PC(PunchVelocitySensing),
-    PC(FilterVelocityScale),
-    PC(FilterVelocityScaleFunction),
+    //Filter
+    rParamZyn(PFilterVelocityScale, "Filter Velocity Magnitude"),
+    rParamZyn(PFilterVelocityScaleFunction, "Filter Velocity Function Shape"),
+
+    //Freq
+    rToggle(Pfixedfreq, "Base frequency fixed frequency enable"),
+    rParamZyn(PfixedfreqET, "Equal temeperate control for fixed frequency operation"),
 
     rParamI(PDetune,        "Fine Detune"),
     rParamI(PCoarseDetune,  "Coarse Detune"),
     rParamZyn(PDetuneType,  "Magnitude of Detune"),
 
-    {"Pbandwidth::i", rProp(parameter) rDoc("Bandwith Of Harmonics"), NULL,
+    {"sample#64:ifb", rProp(internal) rDoc("Nothing to see here"), 0,
+        [](const char *m, rtosc::RtData &d)
+        {
+            PADnoteParameters *p = (PADnoteParameters*)d.obj;
+            const char *mm = m;
+            while(!isdigit(*mm))++mm;
+            unsigned n = atoi(mm);
+            p->sample[n].size     = rtosc_argument(m,0).i;
+            p->sample[n].basefreq = rtosc_argument(m,1).f;
+            p->sample[n].smp      = *(float**)rtosc_argument(m,2).b.data;
+
+            //XXX TODO memory managment (deallocation of smp buffer)
+        }},
+    //weird stuff for PCoarseDetune
+    {"detunevalue:", rMap(unit,cents) rDoc("Get detune value"), NULL,
+        [](const char *, RtData &d)
+        {
+            PADnoteParameters *obj = (PADnoteParameters *)d.obj;
+            d.reply(d.loc, "f", getdetune(obj->PDetuneType, 0, obj->PDetune));
+        }},
+    {"octave::c:i", rProp(parameter) rDoc("Octave note offset"), NULL,
+        [](const char *msg, RtData &d)
+        {
+            PADnoteParameters *obj = (PADnoteParameters *)d.obj;
+            if(!rtosc_narguments(msg)) {
+                int k=obj->PCoarseDetune/1024;
+                if (k>=8) k-=16;
+                d.reply(d.loc, "i", k);
+            } else {
+                int k=(int) rtosc_argument(msg, 0).i;
+                if (k<0) k+=16;
+                obj->PCoarseDetune = k*1024 + obj->PCoarseDetune%1024;
+            }
+        }},
+    {"coarsedetune::c:i", rProp(parameter) rDoc("Coarse note detune"), NULL,
+        [](const char *msg, RtData &d)
+        {
+            PADnoteParameters *obj = (PADnoteParameters *)d.obj;
+            if(!rtosc_narguments(msg)) {
+                int k=obj->PCoarseDetune%1024;
+                if (k>=512) k-=1024;
+                d.reply(d.loc, "i", k);
+            } else {
+                int k=(int) rtosc_argument(msg, 0).i;
+                if (k<0) k+=1024;
+                obj->PCoarseDetune = k + (obj->PCoarseDetune/1024)*1024;
+            }
+        }},
+
+};
+static const rtosc::Ports non_realtime_ports =
+{
+    //Harmonic Source Distribution
+    rRecurp(oscilgen, "Oscillator"),
+    rRecurp(resonance, "Resonance"),
+
+    //Harmonic Shape
+    rOption(Pmode, rMap(min, 0), rMap(max, 2), rOptions(bandwidth,discrete,continious),
+            "Harmonic Distribution Model"),
+    rOption(Php.base.type, rOptions(Gaussian, Rectanglar, Double Exponential),
+            "Harmonic profile shape"),
+    rParamZyn(Php.base.par1, "Harmonic shape distribution parameter"),
+    rParamZyn(Php.freqmult, "Frequency multiplier on distribution"),
+    rParamZyn(Php.modulator.par1, "Distribution modulator parameter"),
+    rParamZyn(Php.modulator.freq, "Frequency of modulator parameter"),
+    rParamZyn(Php.width, "Width of base harmonic"),
+    rOption(Php.amp.mode, rOptions(Sum, Mult, Div1, Div2),
+            "Amplitude harmonic multiplier type"),
+
+    //Harmonic Modulation
+    rOption(Php.amp.type, rOptions(Off, Gauss, Sine, Flat),
+            "Type of amplitude multipler"),
+    rParamZyn(Php.amp.par1, "Amplitude multiplier parameter"),
+    rParamZyn(Php.amp.par2, "Amplitude multiplier parameter"),
+    rToggle(Php.autoscale, "Autoscaling Harmonics"),
+    rOption(Php.onehalf,
+            rOptions(Full, Upper Half, Lower Half),
+            "Harmonic cutoff model"),
+
+    //Harmonic Bandwidth
+    rOption(Pbwscale,
+            rOptions(Normal,
+              EqualHz, Quater,
+              Half, 75%, 150%,
+              Double, Inv. Half),
+            "Bandwidth scaling"),
+
+    //Harmonic Position Modulation
+    rOption(Phrpos.type,
+            rOptions(Harmonic, ShiftU, ShiftL, PowerU, PowerL, Sine,
+                Power, Shift),
+            "Harmonic Overtone shifting mode"),
+    rParamZyn(Phrpos.par1, "Harmonic position parameter"),
+    rParamZyn(Phrpos.par2, "Harmonic position parameter"),
+    rParamZyn(Phrpos.par3, "Harmonic position parameter"),
+
+    //Quality
+    rOption(Pquality.samplesize,
+            rOptions(16k (Tiny), 32k, 64k (Small), 128k,
+              256k (Normal), 512k, 1M (Big)),
+            "Size of each wavetable element"),
+    rOption(Pquality.basenote,
+            rOptions( C-2, G-2, C-3, G-3, C-4,
+                G-4, C-5, G-5, G-6,),
+            "Base note for wavetable"),
+    rOption(Pquality.smpoct,
+            rOptions(0.5, 1, 2, 3, 4, 6, 12),
+            "Samples per octave"),
+    rParamI(Pquality.oct, rLinear(0,7),
+            "Number of octaves to sample (above the first sample"),
+
+    {"Pbandwidth::i", rProp(parameter) rLinear(0,1000) rDoc("Bandwith Of Harmonics"), NULL,
         [](const char *msg, rtosc::RtData &d) {
             PADnoteParameters *p = ((PADnoteParameters*)d.obj);
             if(rtosc_narguments(msg)) {
@@ -114,8 +190,8 @@ static const rtosc::Ports PADnotePorts =
             } else {
                 d.reply(d.loc, "i", p->Pbandwidth);
             }}},
-    
-    {"bandwidthvalue:", NULL, NULL,
+
+    {"bandwidthvalue:", rMap(unit, cents) rDoc("Get Bandwidth"), NULL,
         [](const char *, rtosc::RtData &d) {
             PADnoteParameters *p = ((PADnoteParameters*)d.obj);
             d.reply(d.loc, "f", p->setPbandwidth(p->Pbandwidth));
@@ -143,54 +219,18 @@ static const rtosc::Ports PADnotePorts =
             d.reply(d.loc, "b", n*sizeof(float), tmp);
             d.reply(d.loc, "i", realbw);
             delete[] tmp;}},
-    {"sample#64:ifb", rDoc("Nothing to see here"), 0,
-        [](const char *m, rtosc::RtData &d)
-        {
-            PADnoteParameters *p = (PADnoteParameters*)d.obj;
-            const char *mm = m;
-            while(!isdigit(*mm))++mm;
-            unsigned n = atoi(mm);
-            p->sample[n].size     = rtosc_argument(m,0).i;
-            p->sample[n].basefreq = rtosc_argument(m,1).f;
-            p->sample[n].smp      = *(float**)rtosc_argument(m,2).b.data;
-
-            //XXX TODO memory managment (deallocation of smp buffer)
-        }},
-    //weird stuff for PCoarseDetune
-    {"detunevalue:", NULL, NULL, [](const char *, RtData &d)
-        {
-            PADnoteParameters *obj = (PADnoteParameters *)d.obj;
-            d.reply(d.loc, "f", getdetune(obj->PDetuneType, 0, obj->PDetune));
-        }},
-    {"octave::c:i", NULL, NULL, [](const char *msg, RtData &d)
-        {
-            PADnoteParameters *obj = (PADnoteParameters *)d.obj;
-            if(!rtosc_narguments(msg)) {
-                int k=obj->PCoarseDetune/1024;
-                if (k>=8) k-=16;
-                d.reply(d.loc, "i", k);
-            } else {
-                int k=(int) rtosc_argument(msg, 0).i;
-                if (k<0) k+=16;
-                obj->PCoarseDetune = k*1024 + obj->PCoarseDetune%1024;
-            }
-        }},
-    {"coarsedetune::c:i", NULL, NULL, [](const char *msg, RtData &d)
-        {
-            PADnoteParameters *obj = (PADnoteParameters *)d.obj;
-            if(!rtosc_narguments(msg)) {
-                int k=obj->PCoarseDetune%1024;
-                if (k>=512) k-=1024;
-                d.reply(d.loc, "i", k);
-            } else {
-                int k=(int) rtosc_argument(msg, 0).i;
-                if (k<0) k+=1024;
-                obj->PCoarseDetune = k + (obj->PCoarseDetune/1024)*1024;
-            }
-        }},
 };
 
-const rtosc::Ports &PADnoteParameters::ports = PADnotePorts;
+const rtosc::Ports &PADnoteParameters::non_realtime_ports = ::non_realtime_ports;
+const rtosc::Ports &PADnoteParameters::realtime_ports     = ::realtime_ports;
+
+
+const rtosc::MergePorts PADnoteParameters::ports =
+{
+    &non_realtime_ports,
+    &realtime_ports
+};
+
 
 PADnoteParameters::PADnoteParameters(const SYNTH_T &synth_, FFTwrapper *fft_)
     :Presets(), synth(synth_)
@@ -285,6 +325,7 @@ void PADnoteParameters::defaults()
     PAmpVelocityScaleFunction = 64;
     AmpEnvelope->defaults();
     AmpLfo->defaults();
+    Fadein_adjustment = FADEIN_ADJUSTMENT_SCALE;
     PPunchStrength = 0;
     PPunchTime     = 60;
     PPunchStretch  = 64;
@@ -643,8 +684,9 @@ void PADnoteParameters::generatespectrum_bandwidthMode(float *spectrum,
             const float ibasefreq = realfreq / (synth.samplerate_f * 0.5f) * size;
             for(int i = 0; i < profilesize; ++i) {
                 const float idfreq = (i / (float)profilesize - 0.5f) * ibw;
-                const int   spfreq  = (int) (idfreq + ibasefreq);
-                const float fspfreq = fmodf((float)idfreq + ibasefreq, 1.0f);
+                const float freqsum = idfreq + ibasefreq;
+                const int   spfreq  = (int)freqsum;
+                const float fspfreq = freqsum - spfreq;
                 if(spfreq <= 0)
                     continue;
                 if(spfreq >= size - 1)
@@ -909,6 +951,7 @@ void PADnoteParameters::add2XML(XMLwrapper *xml)
     xml->addpar("volume", PVolume);
     xml->addpar("panning", PPanning);
     xml->addpar("velocity_sensing", PAmpVelocityScaleFunction);
+    xml->addpar("fadein_adjustment", Fadein_adjustment);
     xml->addpar("punch_strength", PPunchStrength);
     xml->addpar("punch_time", PPunchTime);
     xml->addpar("punch_stretch", PPunchStretch);
@@ -1020,6 +1063,7 @@ void PADnoteParameters::getfromXML(XMLwrapper *xml)
         PPanning = xml->getpar127("panning", PPanning);
         PAmpVelocityScaleFunction = xml->getpar127("velocity_sensing",
                                                    PAmpVelocityScaleFunction);
+        Fadein_adjustment = xml->getpar127("fadein_adjustment", Fadein_adjustment);
         PPunchStrength = xml->getpar127("punch_strength", PPunchStrength);
         PPunchTime     = xml->getpar127("punch_time", PPunchTime);
         PPunchStretch  = xml->getpar127("punch_stretch", PPunchStretch);
