@@ -23,38 +23,30 @@
 #ifndef PRESETS_H
 #define PRESETS_H
 
-#include "../Misc/XMLwrapper.h"
-
-#include "PresetsStore.h"
+#include "../globals.h"
+class PresetsStore;
 
 /**Presets and Clipboard management*/
 class Presets
 {
-public:
-    Presets();
-    virtual ~Presets();
+    friend class PresetsArray;
+    public:
+        Presets();
+        virtual ~Presets();
 
-    void copy(const char *name);/**<if name==NULL, the clipboard is used*/
-    void paste(int npreset);//npreset==0 for clipboard
-    bool checkclipboardtype();
-    void deletepreset(int npreset);
+        virtual void copy(PresetsStore &ps, const char *name); /**<if name==NULL, the clipboard is used*/
+        //virtual void paste(PresetsStore &ps, int npreset); //npreset==0 for clipboard
+        virtual bool checkclipboardtype(PresetsStore &ps);
+        void deletepreset(PresetsStore &ps, int npreset);
 
-    char type[MAX_PRESETTYPE_SIZE];
-    void setelement(int n);
-
-    void rescanforpresets();
-
-protected:
-    void setpresettype(const char *type);
-private:
-    virtual void add2XML(XMLwrapper *xml)=0;
-    virtual void getfromXML(XMLwrapper *xml)=0;
-    virtual void defaults()=0;
-    virtual void add2XMLsection(XMLwrapper *xml,int n) {};
-    virtual void getfromXMLsection(XMLwrapper *xml,int n) {};
-    virtual void defaults(int n) {};
-    int nelement;
+        char type[MAX_PRESETTYPE_SIZE];
+        //void setelement(int n);
+    protected:
+        void setpresettype(const char *type);
+    private:
+        virtual void add2XML(XMLwrapper *xml)    = 0;
+        //virtual void getfromXML(XMLwrapper *xml) = 0;
+        //virtual void defaults() = 0;
 };
 
 #endif
-
