@@ -53,6 +53,8 @@ class ADnote:public SynthNote
         int noteout(float *outl, float *outr);
         void releasekey();
         int finished() const;
+
+        virtual SynthNote *cloneLegato(void) override;
     private:
 
         /**Changes the frequency of an oscillator.
@@ -97,7 +99,8 @@ class ADnote:public SynthNote
         inline void ComputeVoiceOscillatorPitchModulation(int nvoice);
 
         /**Generate Noise Samples for Voice*/
-        inline void ComputeVoiceNoise(int nvoice);
+        inline void ComputeVoiceWhiteNoise(int nvoice);
+        inline void ComputeVoicePinkNoise(int nvoice);
 
         /**Fadein in a way that removes clicks but keep sound "punchy"*/
         inline void fadein(float *smps) const;
@@ -119,6 +122,7 @@ class ADnote:public SynthNote
             void kill(Allocator &memory);
             void initparameters(const ADnoteGlobalParam &param,
                                 const SYNTH_T &synth,
+                                const AbsTime &time,
                                 class Allocator &memory,
                                 float basefreq, float velocity,
                                 bool stereo);
@@ -140,6 +144,7 @@ class ADnote:public SynthNote
             Envelope *AmpEnvelope;
             LFO      *AmpLfo;
 
+            float Fadein_adjustment;
             struct {
                 int   Enabled;
                 float initialvalue, dt, t;
@@ -246,8 +251,8 @@ class ADnote:public SynthNote
         /*    INTERNAL VALUES OF THE NOTE AND OF THE VOICES     */
         /********************************************************/
 
-        //time from the start of the note
-        float time;
+	//pinking filter (Paul Kellet)
+	float pinking[NUM_VOICES][14]; 
 
         //the size of unison for a single voice
         int unison_size[NUM_VOICES];
