@@ -38,44 +38,63 @@
 
 pthread_t main_thread;
 
-#define PC(x) rParamZyn(P##x, "undocumented oscilgen parameter")
-
 #define rObject OscilGen
-const rtosc::Ports OscilGen::ports = {
+const rtosc::Ports OscilGen::non_realtime_ports = {
     rSelf(OscilGen),
     rPaste,
-    PC(hmagtype),
-    PC(currentbasefunc),
-    PC(basefuncpar),
-    PC(basefuncmodulation),
-    PC(basefuncmodulationpar1),
-    PC(basefuncmodulationpar2),
-    PC(basefuncmodulationpar3),
-    PC(waveshaping),
-    PC(waveshapingfunction),
-    PC(filtertype),
-    PC(filterpar1),
-    PC(filterpar2),
-    PC(filterbeforews),
-    PC(satype),
-    PC(sapar),
+    //TODO ensure min/max
+    rOption(Phmagtype,
+            rOptions(linear,dB scale (-40),
+                     dB scale (-60), dB scale (-80),
+                     dB scale (-100)),
+            "Type of magnitude for harmonics"),
+    rOption(Pcurrentbasefunc,
+            rOptions(sine, triangle, pulse, saw, power, gauss,
+                diode, abssine, pulsesine, stretchsine,
+                chirp, absstretchsine, chebyshev, sqr,
+                spike, circle), rOpt(127,use-as-base waveform),
+            "Base Waveform for harmonics"),
+    rParamZyn(Pbasefuncpar,
+            "Morph between possible base function shapes "
+            "(e.g. rising sawtooth vs a falling sawtooth)"),
+    rOption(Pbasefuncmodulation,
+            rOptions(None, Rev, Sine, Power, Chop),
+            "Modulation applied to Base function spectra"),
+    rParamZyn(Pbasefuncmodulationpar1,
+            "Base function modulation parameter"),
+    rParamZyn(Pbasefuncmodulationpar2,
+            "Base function modulation parameter"),
+    rParamZyn(Pbasefuncmodulationpar3,
+            "Base function modulation parameter"),
+    rParamZyn(Pwaveshaping, "Degree Of Waveshaping"),
+    rOption(Pwaveshapingfunction,
+            rOptions(Undistorted,
+                Arctangent, Asymmetric, Pow, Sine, Quantisize,
+                Zigzag, Limiter, Upper Limiter, Lower Limiter,
+                Inverse Limiter, Clip, Asym2, Pow2, sigmoid),
+            "Shape of distortion to be applied"),
+    rOption(Pfiltertype, rOptions(No Filter,
+            lp, hp1, hp1b, bp1, bs1, lp2, hp2, bp2, bs2,
+            cos, sin, low_shelf, s), "Harmonic Filter"),
+    rParamZyn(Pfilterpar1, "Filter parameter"),
+    rParamZyn(Pfilterpar2, "Filter parameter"),
+    rToggle(Pfilterbeforews, "Filter before waveshaping spectra;"
+            "When enabled oscilfilter(freqs); then waveshape(freqs);, "
+            "otherwise waveshape(freqs); then oscilfilter(freqs);"),
+    rOption(Psatype, rOptions(None, Pow, ThrsD, ThrsU),
+            "Spectral Adjustment Type"),
+    rParamZyn(Psapar, "Spectral Adjustment Parameter"),
     rParamI(Pharmonicshift, "Amount of shift on harmonics"),
     rToggle(Pharmonicshiftfirst, "If harmonics are shifted before waveshaping/filtering"),
-    PC(modulation),
-    PC(modulationpar1),
-    PC(modulationpar2),
-    PC(modulationpar3),
-    //FIXME realtime parameters lurking below
-    PC(rand),
-    PC(amprandpower),
-    PC(amprandtype),
-    PC(adaptiveharmonics),
-    PC(adaptiveharmonicsbasefreq),
-    PC(adaptiveharmonicspower),
-    PC(adaptiveharmonicspar),
+    rOption(Pmodulation, rOptions(None, Rev, Sine, Power),
+            "Frequency Modulation To Combined Spectra"),
+    rParamZyn(Pmodulationpar1, "modulation parameter"),
+    rParamZyn(Pmodulationpar2, "modulation parameter"),
+    rParamZyn(Pmodulationpar3, "modulation parameter"),
+
 
     //TODO update to rArray and test
-    {"phase#128::c", rDoc("Sets harmonic phase"),
+    {"phase#128::c:i", rProp(parameter) rLinear(0,127) rDoc("Sets harmonic phase"),
         NULL, [](const char *m, rtosc::RtData &d) {
             const char *mm = m;
             while(*mm && !isdigit(*mm)) ++mm;
@@ -86,7 +105,7 @@ const rtosc::Ports OscilGen::ports = {
                 phase = rtosc_argument(m,0).i;
         }},
     //TODO update to rArray and test
-    {"magnitude#128::c", rDoc("Sets harmonic magnitude"),
+    {"magnitude#128::c:i", rProp(parameter) rLinear(0,127) rDoc("Sets harmonic magnitude"),
         NULL, [](const char *m, rtosc::RtData &d) {
             //printf("I'm at '%s'\n", d.loc);
             const char *mm = m;
@@ -94,8 +113,20 @@ const rtosc::Ports OscilGen::ports = {
             unsigned char &mag = ((OscilGen*)d.obj)->Phmag[atoi(mm)];
             if(!rtosc_narguments(m))
                 d.reply(d.loc, "c", mag);
-            else
+            else {
                 mag = rtosc_argument(m,0).i;
+                //printf("setting magnitude\n\n");
+                //XXX hack hack
+                char *repath = strdup(d.loc);
+                char *edit   = rindex(repath, '/')+1;
+                strcpy(edit, "prepare");
+                OscilGen &o = *((OscilGen*)d.obj);
+                fft_t *data = new fft_t[o.synth.oscilsize / 2];
+                o.prepare(data);
+                // fprintf(stderr, "sending '%p' of fft data\n", data);
+                d.chain(repath, "b", sizeof(fft_t*), &data);
+                o.pendingfreqs = data;
+            }
         }},
     {"base-spectrum:", rProp(non-realtime) rDoc("Returns spectrum of base waveshape"),
         NULL, [](const char *, rtosc::RtData &d) {
@@ -117,17 +148,45 @@ const rtosc::Ports OscilGen::ports = {
             d.reply(d.loc, "b", n*sizeof(float), smps);
             delete[] smps;
         }},
-    {"spectrum:", rProp(non-realtime) rDoc("Returns spectrum of waveform"),
+    {"prepare:", rProp(non-realtime) rDoc("Performs setup operation to oscillator"),
         NULL, [](const char *, rtosc::RtData &d) {
-            OscilGen &o = *((OscilGen*)d.obj);
-            const unsigned n = o.synth.oscilsize / 2;
-            float *spc = new float[n];
-            memset(spc, 0, 4*n);
-            ((OscilGen*)d.obj)->getspectrum(n,spc,0);
-            d.reply(d.loc, "b", n*sizeof(float), spc);
-            delete[] spc;
+            //fprintf(stderr, "prepare: got a message from '%s'\n", m);
+            OscilGen &o = *(OscilGen*)d.obj;
+            fft_t *data = new fft_t[o.synth.oscilsize / 2];
+            o.prepare(data);
+            // fprintf(stderr, "sending '%p' of fft data\n", data);
+            d.chain(d.loc, "b", sizeof(fft_t*), &data);
+            o.pendingfreqs = data;
         }},
-    {"waveform:", rProp(non-realtime) rDoc("Returns waveform points"),
+    {"convert2sine:", rProp(non-realtime) rDoc("Translates waveform into FS"),
+        NULL, [](const char *, rtosc::RtData &d) {
+            ((OscilGen*)d.obj)->convert2sine();
+        }},
+    {"use-as-base:", rProp(non-realtime) rDoc("Translates current waveform into base"),
+        NULL, [](const char *, rtosc::RtData &d) {
+            ((OscilGen*)d.obj)->useasbase();
+        }}};
+
+#define rForwardCb [](const char *msg, rtosc::RtData &d) {\
+    printf("fowarding...\n"); d.forward();}
+const rtosc::Ports OscilGen::realtime_ports{
+    rSelf(OscilGen),
+    rParamZyn(Prand, "Oscilator Phase Randomness: smaller than 0 is \""
+            "group\", larger than 0 is for each harmonic"),
+    rParamZyn(Pamprandpower,
+            "Variance of harmonic randomness"),
+    rOption(Pamprandtype, rOptions(None, Pow, Sin),
+            "Harmonic random distribution to select from"),
+    rOption(Padaptiveharmonics,
+            rOptions(OFF, ON, Square, 2xSub, 2xAdd, 3xSub, 3xAdd, 4xSub, 4xAdd),
+            "Adaptive Harmonics Mode"),
+    rParamI(Padaptiveharmonicsbasefreq, rLinear(0,255),
+            "Base frequency of adaptive harmonic (30..3000Hz)"),
+    rParamI(Padaptiveharmonicspower,rLinear(0,200),
+            "Adaptive Harmonic Strength"),
+    rParamZyn(Padaptiveharmonicspar,
+            "Adaptive Harmonics Postprocessing Power"),
+    {"waveform:", rDoc("Returns waveform points"),
         NULL, [](const char *, rtosc::RtData &d) {
             OscilGen &o = *((OscilGen*)d.obj);
             const unsigned n = o.synth.oscilsize;
@@ -139,33 +198,31 @@ const rtosc::Ports OscilGen::ports = {
             d.reply(d.loc, "b", n*sizeof(float), smps);
             delete[] smps;
         }},
-    {"prepare:", rProp(non-realtime) rDoc("Performs setup operation to oscillator"),
+    {"spectrum:", rDoc("Returns spectrum of waveform"),
         NULL, [](const char *, rtosc::RtData &d) {
-            //fprintf(stderr, "prepare: got a message from '%s'\n", m);
-            OscilGen &o = *(OscilGen*)d.obj;
-            fft_t *data = new fft_t[o.synth.oscilsize / 2];
-            o.prepare(data);
-            //fprintf(stderr, "sending '%p' of fft data\n", data);
-            d.reply("/forward", "sb", d.loc, sizeof(fft_t*), &data);
-            o.pendingfreqs = data;
+            OscilGen &o = *((OscilGen*)d.obj);
+            const unsigned n = o.synth.oscilsize / 2;
+            float *spc = new float[n];
+            memset(spc, 0, 4*n);
+            ((OscilGen*)d.obj)->getspectrum(n,spc,0);
+            d.reply(d.loc, "b", n*sizeof(float), spc);
+            delete[] spc;
         }},
-    {"convert2sine:", rProp(non-realtime) rDoc("Translates waveform into FS"),
-        NULL, [](const char *, rtosc::RtData &d) {
-            ((OscilGen*)d.obj)->convert2sine();
-        }},
-    {"prepare:b", rProp(internal) rProp(non-realtime) rProp(pointer) rDoc("Sets prepared fft data"),
+    {"prepare:b", rProp(internal) rProp(realtime) rProp(pointer) rDoc("Sets prepared fft data"),
         NULL, [](const char *m, rtosc::RtData &d) {
-            //fprintf(stderr, "prepare:b got a message from '%s'\n", m);
+            // fprintf(stderr, "prepare:b got a message from '%s'\n", m);
             OscilGen &o = *(OscilGen*)d.obj;
             assert(rtosc_argument(m,0).b.len == sizeof(void*));
             d.reply("/free", "sb", "fft_t", sizeof(void*), &o.oscilFFTfreqs);
-            //fprintf(stderr, "\n\n");
-            //fprintf(stderr, "The ID of this of this thread is: %ld\n", (long)pthread_self());
-            //fprintf(stderr, "o.oscilFFTfreqs = %p\n", o.oscilFFTfreqs);
-            assert(main_thread != pthread_self());
             assert(o.oscilFFTfreqs !=*(fft_t**)rtosc_argument(m,0).b.data);
             o.oscilFFTfreqs = *(fft_t**)rtosc_argument(m,0).b.data;
         }},
+
+};
+
+const rtosc::MergePorts OscilGen::ports{
+    &OscilGen::realtime_ports,
+    &OscilGen::non_realtime_ports
 };
 
 
@@ -253,6 +310,8 @@ OscilGen::OscilGen(const SYNTH_T &synth_, FFTwrapper *fft_, Resonance *res_)
     outoscilFFTfreqs = new fft_t[synth.oscilsize / 2];
     oscilFFTfreqs    = new fft_t[synth.oscilsize / 2];
     basefuncFFTfreqs = new fft_t[synth.oscilsize / 2];
+    cachedbasefunc = new float[synth.oscilsize];
+    cachedbasevalid = false;
     pendingfreqs     = oscilFFTfreqs;
 
     randseed = 1;
@@ -267,6 +326,7 @@ OscilGen::~OscilGen()
     delete[] outoscilFFTfreqs;
     delete[] basefuncFFTfreqs;
     delete[] oscilFFTfreqs;
+    delete[] cachedbasefunc;
 }
 
 
@@ -346,7 +406,6 @@ void OscilGen::convert2sine()
     float  mag[MAX_AD_HARMONICS], phase[MAX_AD_HARMONICS];
     float  oscil[synth.oscilsize];
     fft_t *freqs = new fft_t[synth.oscilsize / 2];
-
     get(oscil, -1.0f);
     FFTwrapper *fft = new FFTwrapper(synth.oscilsize);
     fft->smps2freqs(oscil, freqs);
@@ -367,7 +426,7 @@ void OscilGen::convert2sine()
         float newmag   = mag[i];
         float newphase = phase[i];
 
-        Phmag[i] = (int) ((newmag) * 64.0f) + 64;
+        Phmag[i] = (int) ((newmag) * 63.0f) + 64;
 
         Phphase[i] = 64 - (int) (64.0f * newphase / PI);
         if(Phphase[i] > 127)
@@ -380,78 +439,77 @@ void OscilGen::convert2sine()
     prepare();
 }
 
+float OscilGen::userfunc(float x)
+{
+    if (!fft)
+        return 0;
+    if (!cachedbasevalid) {
+        fft->freqs2smps(basefuncFFTfreqs, cachedbasefunc);
+        cachedbasevalid = true;
+    }
+    return cinterpolate(cachedbasefunc,
+                        synth.oscilsize,
+                        synth.oscilsize * (x + 1) - 1);
+}
+
 /*
  * Get the base function
  */
 void OscilGen::getbasefunction(float *smps)
 {
-    int   i;
     float par = (Pbasefuncpar + 0.5f) / 128.0f;
     if(Pbasefuncpar == 64)
         par = 0.5f;
 
-    float basefuncmodulationpar1 = Pbasefuncmodulationpar1 / 127.0f,
-          basefuncmodulationpar2 = Pbasefuncmodulationpar2 / 127.0f,
-          basefuncmodulationpar3 = Pbasefuncmodulationpar3 / 127.0f;
+    float p1 = Pbasefuncmodulationpar1 / 127.0f,
+          p2 = Pbasefuncmodulationpar2 / 127.0f,
+          p3 = Pbasefuncmodulationpar3 / 127.0f;
 
     switch(Pbasefuncmodulation) {
         case 1:
-            basefuncmodulationpar1 =
-                (powf(2, basefuncmodulationpar1 * 5.0f) - 1.0f) / 10.0f;
-            basefuncmodulationpar3 =
-                floor((powf(2, basefuncmodulationpar3 * 5.0f) - 1.0f));
-            if(basefuncmodulationpar3 < 0.9999f)
-                basefuncmodulationpar3 = -1.0f;
+            p1 = (powf(2, p1 * 5.0f) - 1.0f) / 10.0f;
+            p3 = floor(powf(2, p3 * 5.0f) - 1.0f);
+            if(p3 < 0.9999f)
+                p3 = -1.0f;
             break;
         case 2:
-            basefuncmodulationpar1 =
-                (powf(2, basefuncmodulationpar1 * 5.0f) - 1.0f) / 10.0f;
-            basefuncmodulationpar3 = 1.0f
-                                     + floor((powf(2, basefuncmodulationpar3
-                                                   * 5.0f) - 1.0f));
+            p1 = (powf(2, p1 * 5.0f) - 1.0f) / 10.0f;
+            p3 = 1.0f + floor(powf(2, p3 * 5.0f) - 1.0f);
             break;
         case 3:
-            basefuncmodulationpar1 =
-                (powf(2, basefuncmodulationpar1 * 7.0f) - 1.0f) / 10.0f;
-            basefuncmodulationpar3 = 0.01f
-                                     + (powf(2, basefuncmodulationpar3
-                                             * 16.0f) - 1.0f) / 10.0f;
+            p1 = (powf(2, p1 * 7.0f) - 1.0f) / 10.0f;
+            p3 = 0.01f + (powf(2, p3 * 16.0f) - 1.0f) / 10.0f;
             break;
     }
 
     base_func func = getBaseFunction(Pcurrentbasefunc);
 
-    for(i = 0; i < synth.oscilsize; ++i) {
+    for(int i = 0; i < synth.oscilsize; ++i) {
         float t = i * 1.0f / synth.oscilsize;
 
         switch(Pbasefuncmodulation) {
-            case 1:
-                t = t * basefuncmodulationpar3 + sinf(
-                    (t
-                     + basefuncmodulationpar2) * 2.0f
-                    * PI) * basefuncmodulationpar1;                              //rev
+            case 1: //rev
+                t = t * p3 + sinf((t + p2) * 2.0f * PI) * p1;
                 break;
-            case 2:
-                t = t + sinf(
-                    (t * basefuncmodulationpar3
-                     + basefuncmodulationpar2) * 2.0f
-                    * PI) * basefuncmodulationpar1;                              //sine
+            case 2: //sine
+                t += sinf( (t * p3 + p2) * 2.0f * PI) * p1;
                 break;
-            case 3:
-                t = t + powf((1.0f - cosf(
-                                  (t
-                                   + basefuncmodulationpar2) * 2.0f
-                                  * PI)) * 0.5f,
-                             basefuncmodulationpar3) * basefuncmodulationpar1; //power
+            case 3: //power
+                t += powf((1.0f - cosf((t + p2) * 2.0f * PI)) * 0.5f, p3) * p1;
                 break;
+            case 4: //chop
+                t = t * (powf(2.0, Pbasefuncmodulationpar1/32.0 +
+                              Pbasefuncmodulationpar2/2048.0)) + p3;
         }
 
         t = t - floor(t);
 
         if(func)
             smps[i] = func(t, par);
-        else
+        else if (Pcurrentbasefunc == 0)
             smps[i] = -sinf(2.0f * PI * i / synth.oscilsize);
+        else
+            smps[i] = userfunc(t);
     }
 }
 
@@ -482,7 +540,8 @@ void OscilGen::changebasefunction(void)
 {
     if(Pcurrentbasefunc != 0) {
         getbasefunction(tmpsmps);
-        fft->smps2freqs(tmpsmps, basefuncFFTfreqs);
+        if(fft)
+            fft->smps2freqs(tmpsmps, basefuncFFTfreqs);
         clearDC(basefuncFFTfreqs);
     }
     else //in this case basefuncFFTfreqs are not used
@@ -651,12 +710,11 @@ void OscilGen::spectrumadjust(fft_t *freqs)
             break;
     }
 
-
     normalize(freqs, synth.oscilsize);
 
     for(int i = 0; i < synth.oscilsize / 2; ++i) {
-        float mag   = abs(oscilFFTfreqs, i);
-        float phase = M_PI_2 - arg(oscilFFTfreqs, i);
+        float mag   = abs(freqs, i);
+        float phase = M_PI_2 - arg(freqs, i);
 
         switch(Psatype) {
             case 1:
@@ -782,13 +840,12 @@ void OscilGen::prepare(fft_t *freqs)
     if(Pharmonicshiftfirst != 0)
         shiftharmonics(freqs);
 
-    if(Pfilterbeforews == 0) {
-        waveshape(freqs);
-        oscilfilter(freqs);
-    }
-    else {
+    if(Pfilterbeforews) {
         oscilfilter(freqs);
         waveshape(freqs);
+    } else {
+        waveshape(freqs);
+        oscilfilter(freqs);
     }
 
     modulation(freqs);
@@ -875,9 +932,9 @@ void OscilGen::adaptiveharmonicpostprocess(fft_t *f, int size)
     if(Padaptiveharmonics == 2) { //2n+1
         for(int i = 0; i < size; ++i)
             if((i % 2) == 0)
-                f[i] += inf[i];  //i=0 pt prima armonica,etc.
+                f[i] += inf[i];  //i=0 first harmonic,etc.
     }
-    else {  //celelalte moduri
+    else {  //other ways
         int nh = (Padaptiveharmonics - 3) / 2 + 2;
         int sub_vs_add = (Padaptiveharmonics - 3) % 2;
         if(sub_vs_add == 0) {
@@ -1072,14 +1129,15 @@ void OscilGen::getspectrum(int n, float *spc, int what)
 
     for(int i = 1; i < n; ++i) {
         if(what == 0)
-            spc[i - 1] = abs(pendingfreqs, i);
+            spc[i] = abs(pendingfreqs, i);
         else {
             if(Pcurrentbasefunc == 0)
-                spc[i - 1] = ((i == 1) ? (1.0f) : (0.0f));
+                spc[i] = ((i == 1) ? (1.0f) : (0.0f));
             else
-                spc[i - 1] = abs(basefuncFFTfreqs, i);
+                spc[i] = abs(basefuncFFTfreqs, i);
         }
     }
+    spc[0]=0;
 
     if(what == 0) {
         for(int i = 0; i < n; ++i)
@@ -1104,6 +1162,7 @@ void OscilGen::useasbase()
 
     oldbasefunc = Pcurrentbasefunc = 127;
     prepare();
+    cachedbasevalid = false;
 }
 
 
@@ -1118,29 +1177,55 @@ void OscilGen::getcurrentbasefunction(float *smps)
         getbasefunction(smps);   //the sine case
 }
 
-#define PRESERVE(x) decltype(this->x) x = this->x
-#define RESTORE(x)  this->x = x
+#define COPY(y) this->y = o.y
 void OscilGen::paste(OscilGen &o)
 {
     //XXX Figure out a better implementation of this sensitive to RT issues...
-    //Preserve Pointer Elements
-    PRESERVE(oscilFFTfreqs);
-    PRESERVE(pendingfreqs);
-    PRESERVE(tmpsmps);
-    PRESERVE(outoscilFFTfreqs);
-    PRESERVE(fft);
-    PRESERVE(basefuncFFTfreqs);
-    PRESERVE(res);
-    memcpy((char*)this, (char*)&o, sizeof(*this));
-    RESTORE(oscilFFTfreqs);
-    RESTORE(pendingfreqs);
-    RESTORE(tmpsmps);
-    RESTORE(outoscilFFTfreqs);
-    RESTORE(fft);
-    RESTORE(basefuncFFTfreqs);
-    RESTORE(res);
+    for(int i=0; i<MAX_AD_HARMONICS; ++i) {
+        COPY(Phmag[i]);
+        COPY(Phphase[i]);
+    }
+
+    COPY(Phmagtype);
+    COPY(Pcurrentbasefunc);
+    COPY(Pbasefuncpar);
+
+    COPY(Pbasefuncmodulation);
+    COPY(Pbasefuncmodulationpar1);
+    COPY(Pbasefuncmodulationpar2);
+    COPY(Pbasefuncmodulationpar3);
+
+    COPY(Pwaveshaping);
+    COPY(Pwaveshapingfunction);
+    COPY(Pfiltertype);
+    COPY(Pfilterpar1);
+    COPY(Pfilterpar2);
+    COPY(Pfilterbeforews);
+    COPY(Psatype);
+    COPY(Psapar);
+
+    COPY(Pharmonicshift);
+    COPY(Pharmonicshiftfirst);
+
+    COPY(Pmodulation);
+    COPY(Pmodulationpar1);
+    COPY(Pmodulationpar2);
+    COPY(Pmodulationpar3);
+
+    COPY(Prand);
+    COPY(Pamprandpower);
+    COPY(Pamprandtype);
+    COPY(Padaptiveharmonics);
+    COPY(Padaptiveharmonicsbasefreq);
+    COPY(Padaptiveharmonicspower);
+    COPY(Padaptiveharmonicspar);
+
+
+    if(this->Pcurrentbasefunc)
+        changebasefunction();
     this->prepare();
 }
+#undef COPY
 
 void OscilGen::add2XML(XMLwrapper *xml)
 {
@@ -1198,7 +1283,7 @@ void OscilGen::add2XML(XMLwrapper *xml)
         for(int i = 1; i < synth.oscilsize / 2; ++i) {
             float xc = basefuncFFTfreqs[i].real();
             float xs = basefuncFFTfreqs[i].imag();
-            if((fabs(xs) > 1e-6f) && (fabs(xc) > 1e-6f)) {
+            if((fabs(xs) > 1e-6f) || (fabs(xc) > 1e-6f)) {
                 xml->beginbranch("BF_HARMONIC", i);
                 xml->addparreal("cos", xc);
                 xml->addparreal("sin", xs);
@@ -1288,7 +1373,6 @@ void OscilGen::getfromXML(XMLwrapper *xml)
     if(Pcurrentbasefunc != 0)
         changebasefunction();
 
-
     if(xml->enterbranch("BASE_FUNCTION")) {
         for(int i = 1; i < synth.oscilsize / 2; ++i)
             if(xml->enterbranch("BF_HARMONIC", i)) {
@@ -1301,8 +1385,8 @@ void OscilGen::getfromXML(XMLwrapper *xml)
 
         clearDC(basefuncFFTfreqs);
         normalize(basefuncFFTfreqs, synth.oscilsize);
-    }
-}
+        cachedbasevalid = false;
+    }}
 
 
 //Define basic functions
