@@ -28,9 +28,9 @@
 #include "../Params/FilterParams.h"
 #include "../Misc/Util.h"
 
-PADnote::PADnote(PADnoteParameters *parameters,
-                 SynthParams pars)
-    :SynthNote(pars), pars(*parameters)
+PADnote::PADnote(const PADnoteParameters *parameters,
+                 SynthParams pars, const int& interpolation)
+    :SynthNote(pars), pars(*parameters), interpolation(interpolation)
 {
     firsttime = true;
     setup(pars.frequency, pars.velocity, pars.portamento, pars.note);
@@ -114,6 +114,9 @@ void PADnote::setup(float freq,
                                               PFilterVelocityScaleFunction) - 1);
 
     if(!legato) {
+        NoteGlobalPar.Fadein_adjustment =
+            pars.Fadein_adjustment / (float)FADEIN_ADJUSTMENT_SCALE;
+        NoteGlobalPar.Fadein_adjustment *= NoteGlobalPar.Fadein_adjustment;
         if(pars.PPunchStrength != 0) {
             NoteGlobalPar.Punch.Enabled = 1;
             NoteGlobalPar.Punch.t = 1.0f; //start from 1.0f and to 0.0f
@@ -121,9 +124,9 @@ void PADnote::setup(float freq,
                 ((powf(10, 1.5f * pars.PPunchStrength / 127.0f) - 1.0f)
                  * VelF(velocity,
                         pars.PPunchVelocitySensing));
-            float time =
+            const float time =
                 powf(10, 3.0f * pars.PPunchTime / 127.0f) / 10000.0f;             //0.1f .. 100 ms
-            float stretch = powf(440.0f / freq, pars.PPunchStretch / 64.0f);
+            const float stretch = powf(440.0f / freq, pars.PPunchStretch / 64.0f);
             NoteGlobalPar.Punch.dt = 1.0f
                                      / (time * synth.samplerate_f * stretch);
         }
@@ -131,10 +134,10 @@ void PADnote::setup(float freq,
             NoteGlobalPar.Punch.Enabled = 0;
 
         NoteGlobalPar.FreqEnvelope = memory.alloc<Envelope>(*pars.FreqEnvelope, basefreq, synth.dt());
-        NoteGlobalPar.FreqLfo      = memory.alloc<LFO>(*pars.FreqLfo, basefreq, synth.dt());
+        NoteGlobalPar.FreqLfo      = memory.alloc<LFO>(*pars.FreqLfo, basefreq, time);
 
         NoteGlobalPar.AmpEnvelope = memory.alloc<Envelope>(*pars.AmpEnvelope, basefreq, synth.dt());
-        NoteGlobalPar.AmpLfo      = memory.alloc<LFO>(*pars.AmpLfo, basefreq, synth.dt());
+        NoteGlobalPar.AmpLfo      = memory.alloc<LFO>(*pars.AmpLfo, basefreq, time);
     }
 
     NoteGlobalPar.Volume = 4.0f
@@ -154,7 +157,7 @@ void PADnote::setup(float freq,
                     synth.samplerate, synth.buffersize);
 
         NoteGlobalPar.FilterEnvelope = memory.alloc<Envelope>(*pars.FilterEnvelope, basefreq, synth.dt());
-        NoteGlobalPar.FilterLfo      = memory.alloc<LFO>(*pars.FilterLfo, basefreq, synth.dt());
+        NoteGlobalPar.FilterLfo      = memory.alloc<LFO>(*pars.FilterLfo, basefreq, time);
     }
     NoteGlobalPar.FilterQ = pars.GlobalFilter->getq();
     NoteGlobalPar.FilterFreqTracking = pars.GlobalFilter->getfreqtracking(
@@ -166,6 +169,13 @@ void PADnote::setup(float freq,
     }
 }
 
+SynthNote *PADnote::cloneLegato(void)
+{
+    SynthParams sp{memory, ctl, synth, time, legato.param.freq, velocity, 
+                   (bool)portamento, legato.param.midinote, true};
+    return memory.alloc<PADnote>(&pars, sp, interpolation);
+}
+
 void PADnote::legatonote(LegatoParams pars)
 {
     // Manage legato stuff
@@ -199,6 +209,7 @@ inline void PADnote::fadein(float *smps)
     float tmp = (synth.buffersize_f - 1.0f) / (zerocrossings + 1) / 3.0f;
     if(tmp < 8.0f)
         tmp = 8.0f;
+    tmp *= NoteGlobalPar.Fadein_adjustment;
 
     int n;
     F2I(tmp, n); //how many samples is the fade-in
@@ -347,7 +358,7 @@ int PADnote::noteout(float *outl, float *outr)
     float freqlo  = freqrap - floor(freqrap);
 
 
-    if(config.cfg.Interpolation)
+    if(interpolation)
         Compute_Cubic(outl, outr, freqhi, freqlo);
     else
         Compute_Linear(outl, outr, freqhi, freqlo);
