@@ -44,7 +44,7 @@ const rtosc::Ports Resonance::ports = {
     rAction(smooth, "Smooth out frequency response"),
     rAction(zero,   "Reset frequency response"),
     //UI Value listeners
-    {"centerfreq:", rDoc("Get center frequency"),  NULL,
+    {"centerfreq:", rDoc("Get center frequency") rMap(unit, Hz),  NULL,
         [](const char *, RtData &d)
         {d.reply(d.loc, "f", ((rObject*)d.obj)->getcenterfreq());}},
     {"octavesfreq:", rDoc("Get center freq of graph"), NULL,
@@ -251,10 +251,21 @@ void Resonance::sendcontroller(MidiControllers ctl, float par)
         ctlbw = par;
 }
 
+#define COPY(y) this->y = r.y
 void Resonance::paste(Resonance &r)
 {
-    memcpy((char*)this, (char*)&r, sizeof(r));
+    COPY(Penabled);
+    for(int i=0; i<N_RES_POINTS; ++i)
+        this->Prespoints[i] = r.Prespoints[i];
+    COPY(PmaxdB);
+    COPY(Pcenterfreq);
+    COPY(Poctavesfreq);
+    COPY(Pprotectthefundamental);
+
+    COPY(ctlcenter);
+    COPY(ctlbw);
 }
+#undef COPY
 
 void Resonance::add2XML(XMLwrapper *xml)
 {
