@@ -13,17 +13,18 @@
 #define ENV_ADSR_FILTER 4
 #define ENV_ADSR_BW 5
 
-class EnvelopeFreeEdit : public Fl_Box, Fl_Osc_Widget
+class EnvelopeFreeEdit : public Fl_Box, public Fl_Osc_Widget
 {
     public:
         EnvelopeFreeEdit(int x,int y, int w, int h, const char *label=0);
         void init(void);
         void setpair(Fl_Box *pair_);
-        int handle(int event);
+        int handle(int event) override;
 
-        void draw(void);
+        void draw(void) override;
         void OSC_raw(const char *msg) override;
-        void update(void);
+        void update(void) override;
+        void rebase(std::string new_base) override;
 
 
         int lastpoint;
