@@ -76,7 +76,7 @@ NSM_Client *nsm = 0;
 
 char *instance_name = 0;
 
-void exitprogram();
+void exitprogram(const Config &config);
 
 extern pthread_t main_thread;
 
@@ -91,21 +91,21 @@ void sigterm_exit(int /*sig*/)
 /*
  * Program initialisation
  */
-void initprogram(SYNTH_T synth, int prefered_port)
+void initprogram(SYNTH_T synth, Config* config, int prefered_port)
 {
-    middleware = new MiddleWare(synth, prefered_port);
+    middleware = new MiddleWare(std::move(synth), config, prefered_port);
     master = middleware->spawnMaster();
     master->swaplr = swaplr;
 
     signal(SIGINT, sigterm_exit);
     signal(SIGTERM, sigterm_exit);
-    Nio::init(master->synth, master);
+    Nio::init(master->synth, config->cfg.oss_devs, master);
 }
 
 /*
  * Program exit
  */
-void exitprogram()
+void exitprogram(const Config& config)
 {
     Nio::stop();
     config.save();
@@ -121,7 +121,6 @@ void exitprogram()
         delete nsm;
 #endif
 
-    delete [] denormalkillbuf;
     FFT_cleanup();
 }
 
@@ -129,16 +128,17 @@ int main(int argc, char *argv[])
 {
     main_thread = pthread_self();
     SYNTH_T synth;
+    Config config;
     config.init();
     int noui = 0;
     cerr
     << "\nZynAddSubFX - Copyright (c) 2002-2013 Nasca Octavian Paul and others"
     << endl;
     cerr
-    << "                Copyright (c) 2009-2014 Mark McCurry [active maintainer]"
+    << "                Copyright (c) 2009-2015 Mark McCurry [active maintainer]"
     << endl;
     cerr << "Compiled: " << __DATE__ << " " << __TIME__ << endl;
-    cerr << "This program is free software (GNU GPL v.2 or later) and \n";
+    cerr << "This program is free software (GNU GPL v2 or later) and \n";
     cerr << "it comes with ABSOLUTELY NO WARRANTY.\n" << endl;
     if(argc == 1)
         cerr << "Try 'zynaddsubfx --help' for command-line options." << endl;
@@ -211,6 +211,9 @@ int main(int argc, char *argv[])
         {
             "dump-oscdoc", 2, NULL, 'd'
         },
+        {
+            "ui-title", 1, NULL, 'u'
+        },
         {
             0, 0, 0, 0
         }
@@ -219,7 +222,7 @@ int main(int argc, char *argv[])
     int option_index = 0, opt, exitwithhelp = 0, exitwithversion = 0;
     int prefered_port = -1;
 
-    string loadfile, loadinstrument, execAfterInit;
+    string loadfile, loadinstrument, execAfterInit, ui_title;
 
     while(1) {
         int tmp = 0;
@@ -227,7 +230,7 @@ int main(int argc, char *argv[])
         /**\todo check this process for a small memory leak*/
         opt = getopt_long(argc,
                           argv,
-                          "l:L:r:b:o:I:O:N:e:P:hvapSDUY",
+                          "l:L:r:b:o:I:O:N:e:P:u:hvapSDUY",
                           opts,
                           &option_index);
         char *optarguments = optarg;
@@ -335,6 +338,10 @@ int main(int argc, char *argv[])
                     outfile << s;
                 }
                 break;
+            case 'u':
+                if(optarguments)
+                    ui_title = optarguments;
+                break;
             case '?':
                 cerr << "ERROR:Bad option or parameter.\n" << endl;
                 exitwithhelp = 1;
@@ -370,32 +377,41 @@ int main(int argc, char *argv[])
              << "  -I , --input\t\t\t\t Set Input Engine\n"
              << "  -e , --exec-after-init\t\t Run post-initialization script\n"
              << "  -d , --dump-oscdoc=FILE\t\t Dump oscdoc xml to file\n"
+             << "  -u , --ui-title=TITLE\t\t Extend UI Window Titles\n"
              << endl;
 
         return 0;
     }
 
-    //produce denormal buf
-    denormalkillbuf = new float [synth.buffersize];
-    for(int i = 0; i < synth.buffersize; ++i)
-        denormalkillbuf[i] = (RND - 0.5f) * 1e-16;
+    cerr.precision(1);
+    cerr << std::fixed;
+    cerr << "\nSample Rate = \t\t" << synth.samplerate << endl;
+    cerr << "Sound Buffer Size = \t" << synth.buffersize << " samples" << endl;
+    cerr << "Internal latency = \t" << synth.dt() * 1000.0f << " ms" << endl;
+    cerr << "ADsynth Oscil.Size = \t" << synth.oscilsize << " samples" << endl;
 
-    initprogram(synth, prefered_port);
+    initprogram(std::move(synth), &config, prefered_port);
 
+    bool altered_master = false;
     if(!loadfile.empty()) {
-        int tmp = master->loadXML(loadfile.c_str());
+        altered_master = true;
+        const char *filename = loadfile.c_str();
+        int tmp = master->loadXML(filename);
         if(tmp < 0) {
             cerr << "ERROR: Could not load master file " << loadfile
                  << "." << endl;
             exit(1);
         }
         else {
+            strncpy(master->last_xmz, filename, XMZ_PATH_MAX);
+            master->last_xmz[XMZ_PATH_MAX-1] = 0;
             master->applyparameters();
             cout << "Master file loaded." << endl;
         }
     }
 
     if(!loadinstrument.empty()) {
+        altered_master = true;
         int loadtopart = 0;
         int tmp = master->part[loadtopart]->loadXMLinstrument(
             loadinstrument.c_str());
@@ -411,15 +427,11 @@ int main(int argc, char *argv[])
         }
     }
 
+    if(altered_master)
+        middleware->updateResources(master);
+
     //Run the Nio system
     bool ioGood = Nio::start();
-    
-    cerr.precision(1);
-    cerr << std::fixed;
-    cerr << "\nSample Rate = \t\t" << synth.samplerate << endl;
-    cerr << "Sound Buffer Size = \t" << synth.buffersize << " samples" << endl;
-    cerr << "Internal latency = \t" << synth.dt() * 1000.0f << " ms" << endl;
-    cerr << "ADsynth Oscil.Size = \t" << synth.oscilsize << " samples" << endl;
 
     if(!execAfterInit.empty()) {
         cout << "Executing user supplied command: " << execAfterInit << endl;
@@ -452,6 +464,10 @@ int main(int argc, char *argv[])
         delete [] msg;
     }
 
+    //set titles
+    if(!ui_title.empty())
+        GUI::raiseUi(gui, "/ui/title", "s", ui_title.c_str());
+
     if(!noui)
     {
         GUI::raiseUi(gui, "/show",  "i", config.cfg.UserInterfaceMode);
@@ -519,6 +535,6 @@ done:
         middleware->tick();
     }
 
-    exitprogram();
+    exitprogram(config);
     return 0;
 }
